@@ -1,43 +1,43 @@
 # -*- coding: UTF-8 -*-
 
+import asyncio
+import datetime
 import os
 import webbrowser
 import math
-import atexit
-import logging
-import signal
-import subprocess
 import platform
 import sys
+import subprocess
+from urllib.request import urlopen, url2pathname, pathname2url
+from urllib.parse import unquote
 
 from gi.repository import Gdk
+from gi.repository import Gio
 from gi.repository import Gtk
 from gi.repository import GLib
 
-from pychess.compat import urlopen, basestring, open, pathname2url, url2pathname, unquote
-from pychess.System import conf, uistuff, prefix, SubProcess, idle_add
-from pychess.System.uistuff import POSITION_GOLDEN
-from pychess.System.Log import log, LogPipe
-from pychess.System.LogEmitter import GLogHandler, logemitter
-from pychess.System.debug import start_thread_dump, print_obj_referrers, print_muppy_sumary
-from pychess.System.prefix import getUserDataPrefix, addUserDataPrefix
+from pychess.System.Log import log
+from pychess.System import conf, uistuff, prefix
+from pychess.System.debug import print_obj_referrers, print_muppy_sumary
 from pychess.Utils.const import HINT, NAME, SPY
-from pychess.Utils.checkversion import checkversion
+# from pychess.Utils.checkversion import checkversion
 from pychess.widgets import enginesDialog
 from pychess.widgets import newGameDialog
-from pychess.widgets import tipOfTheDay
+from pychess.widgets.tipOfTheDay import TipOfTheDay
 from pychess.widgets.discovererDialog import DiscovererDialog
+from pychess.widgets.ExternalsDialog import ExternalsDialog
 from pychess.widgets import gamewidget
-from pychess.widgets.ionest import game_handler
-from pychess.widgets import analyzegameDialog
+from pychess.widgets.analyzegameDialog import AnalyzeGameDialog
 from pychess.widgets import preferencesDialog, gameinfoDialog, playerinfoDialog
-from pychess.widgets.TaskerManager import tasker
-from pychess.widgets.TaskerManager import NewGameTasker
-from pychess.widgets.TaskerManager import InternetGameTasker
+from pychess.widgets.TaskerManager import internet_game_tasker
 from pychess.Players.engineNest import discoverer
 from pychess.Savers import chesspastebin
 from pychess.ic import ICLogon
-# from pychess.Database.gamelist import GameList
+from pychess.perspectives import perspective_manager
+from pychess.perspectives.welcome import Welcome
+from pychess.perspectives.games import Games, get_open_dialog
+from pychess.perspectives.fics import FICS
+from pychess.perspectives.database import Database
 from pychess import VERSION, VERSION_NAME
 
 leftkeys = list(map(Gdk.keyval_from_name, ("Left", "KP_Left")))
@@ -59,7 +59,10 @@ DND_LIST = [Gtk.TargetEntry.new("text/uri-list", DRAG_RESTRICT, TARGET_TYPE_URI_
 
 
 class GladeHandlers(object):
-    def on_window_key_press(window, event):
+    def __init__(self, app):
+        self.app = app
+
+    def on_window_key_press(self, window, event):
         log.debug('on_window_key_press: %s %s' % (window.get_title(), event))
 
         # debug leaking memory
@@ -70,42 +73,42 @@ class GladeHandlers(object):
                 print_obj_referrers()
 
         # Tabbing related shortcuts
-        if not gamewidget.getheadbook():
+        persp = perspective_manager.get_perspective("games")
+        if not persp.getheadbook():
             pagecount = 0
         else:
-            pagecount = gamewidget.getheadbook().get_n_pages()
+            pagecount = persp.getheadbook().get_n_pages()
         if pagecount > 1:
             if event.get_state() & Gdk.ModifierType.CONTROL_MASK:
-                page_num = gamewidget.getheadbook().get_current_page()
+                page_num = persp.getheadbook().get_current_page()
                 # Move selected
                 if event.get_state() & Gdk.ModifierType.SHIFT_MASK:
-                    child = gamewidget.getheadbook().get_nth_page(page_num)
+                    child = persp.getheadbook().get_nth_page(page_num)
                     if event.keyval == Gdk.KEY_Page_Up:
-                        gamewidget.getheadbook().reorder_child(child, (
+                        persp.getheadbook().reorder_child(child, (
                             page_num - 1) % pagecount)
                         return True
                     elif event.keyval == Gdk.KEY_Page_Down:
-                        gamewidget.getheadbook().reorder_child(child, (
+                        persp.getheadbook().reorder_child(child, (
                             page_num + 1) % pagecount)
                         return True
                 # Change selected
                 else:
                     if event.keyval == Gdk.KEY_Page_Up:
-                        gamewidget.getheadbook().set_current_page(
+                        persp.getheadbook().set_current_page(
                             (page_num - 1) % pagecount)
                         return True
                     elif event.keyval == Gdk.KEY_Page_Down:
-                        gamewidget.getheadbook().set_current_page(
+                        persp.getheadbook().set_current_page(
                             (page_num + 1) % pagecount)
                         return True
 
-        gmwidg = gamewidget.cur_gmwidg()
+        gmwidg = persp.cur_gmwidg()
         if gmwidg is not None:
-            for panel in gmwidg.panels:
-                focused = panel.get_focus_child()
-                # Do nothing in chat panel
-                if focused is not None and isinstance(focused, Gtk.Entry):
-                    return False
+            # Let default handler work if typing inside entry widgets
+            current_focused_widget = gamewidget.getWidgets()["main_window"].get_focus()
+            if current_focused_widget is not None and isinstance(current_focused_widget, Gtk.Entry):
+                return False
 
             # Navigate on boardview with arrow keys
             if event.keyval in leftkeys:
@@ -145,21 +148,21 @@ class GladeHandlers(object):
             return False
 
     def on_recent_game_activated(self, uri):
-        if isinstance(uri, basestring):
+        if isinstance(uri, str):
             path = url2pathname(uri)
             recentManager.add_item("file:" + pathname2url(path))
 
-    #          Drag 'n' Drop          #
+    # Drag 'n' Drop
 
-    def on_drag_received(self, widget, context, x, y, selection, target_type,
-                         timestamp):
+    def on_drag_received(self, widget, context, x, y, selection, target_type, timestamp):
         if target_type == TARGET_TYPE_URI_LIST:
             uris = selection.get_uris()
-            if len(uris) == 1 and uris[0].lower().endswith(".pgn"):
-                uri = uris[0]
-                newGameDialog.LoadFileExtension.run(uri)
-            else:
-                newGameDialog.loadFilesAndRun(uris)
+            for uri in uris:
+                if uri.lower().endswith(".fen"):
+                    newGameDialog.loadFileAndRun(uri)
+                else:
+                    perspective = perspective_manager.get_perspective("database")
+                    perspective.open_chessfile(uri)
 
     # Game Menu
 
@@ -170,12 +173,29 @@ class GladeHandlers(object):
         ICLogon.run()
 
     def on_load_game1_activate(self, widget):
-        newGameDialog.LoadFileExtension.run(None)
+        dialog = get_open_dialog()
+
+        response = dialog.run()
+        if response == Gtk.ResponseType.OK:
+            filenames = dialog.get_filenames()
+        else:
+            filenames = None
+
+        dialog.destroy()
+
+        if filenames is not None:
+            for filename in filenames:
+                if filename.lower().endswith(".fen"):
+                    newGameDialog.loadFileAndRun(filename)
+                else:
+                    perspective = perspective_manager.get_perspective("database")
+                    perspective.open_chessfile(filename)
 
     def on_set_up_position_activate(self, widget):
         rotate_menu = gamewidget.getWidgets()["rotate_board1"]
         rotate_menu.set_sensitive(True)
-        gmwidg = gamewidget.cur_gmwidg()
+        persp = perspective_manager.get_perspective("games")
+        gmwidg = persp.cur_gmwidg()
         if gmwidg is not None:
             if len(gmwidg.gamemodel.boards) == 1:
                 ply = 0
@@ -186,34 +206,36 @@ class GladeHandlers(object):
             fen = None
         newGameDialog.SetupPositionExtension.run(fen)
 
-    def on_open_database_activate(self, widget):
-        # GameList().load_games()
-        pass
-
     def on_enter_game_notation_activate(self, widget):
         newGameDialog.EnterNotationExtension.run()
 
     def on_save_game1_activate(self, widget):
-        gmwidg = gamewidget.cur_gmwidg()
+        perspective = perspective_manager.get_perspective("games")
+        gmwidg = perspective.cur_gmwidg()
         position = gmwidg.board.view.shown
-        game_handler.saveGame(gmwidg.gamemodel, position)
+        perspective.saveGame(gmwidg.gamemodel, position)
 
     def on_save_game_as1_activate(self, widget):
-        gmwidg = gamewidget.cur_gmwidg()
+        perspective = perspective_manager.get_perspective("games")
+        gmwidg = perspective.cur_gmwidg()
         position = gmwidg.board.view.shown
-        game_handler.saveGameAs(gmwidg.gamemodel, position)
+        perspective.saveGameAs(gmwidg.gamemodel, position)
 
     def on_share_game_activate(self, widget):
-        gmwidg = gamewidget.cur_gmwidg()
+        perspective = perspective_manager.get_perspective("games")
+        gmwidg = perspective.cur_gmwidg()
         chesspastebin.paste(gmwidg.gamemodel)
 
     def on_export_position_activate(self, widget):
-        gmwidg = gamewidget.cur_gmwidg()
+        perspective = perspective_manager.get_perspective("games")
+        gmwidg = perspective.cur_gmwidg()
         position = gmwidg.board.view.shown
-        game_handler.saveGameAs(gmwidg.gamemodel, position, export=True)
+        perspective = perspective_manager.get_perspective("games")
+        perspective.saveGameAs(gmwidg.gamemodel, position, export=True)
 
     def on_analyze_game_activate(self, widget):
-        analyzegameDialog.run()
+        analyze_game_dialog = AnalyzeGameDialog()
+        analyze_game_dialog.run()
 
     def on_properties1_activate(self, widget):
         gameinfoDialog.run(gamewidget.getWidgets())
@@ -222,21 +244,28 @@ class GladeHandlers(object):
         playerinfoDialog.run(gamewidget.getWidgets())
 
     def on_close1_activate(self, widget):
-        gmwidg = gamewidget.cur_gmwidg()
-        game_handler.closeGame(gmwidg)
+        persp = perspective_manager.current_perspective
+        if persp.name == "games":
+            gmwidg = persp.cur_gmwidg()
+            persp.closeGame(gmwidg)
+        elif persp.name == "database":
+            persp.close()
 
     def on_quit1_activate(self, widget, *args):
+        perspective = perspective_manager.get_perspective("games")
         if isinstance(widget, Gdk.Event):
-            if len(game_handler.gamewidgets) == 1 and conf.get("hideTabs", False):
-                gmwidg = gamewidget.cur_gmwidg()
-                game_handler.closeGame(gmwidg, gmwidg.gamemodel)
+            if len(perspective.gamewidgets) == 1 and conf.get("hideTabs", False):
+                gmwidg = perspective.cur_gmwidg()
+                perspective.closeGame(gmwidg, gmwidg.gamemodel)
                 return True
-            elif len(game_handler.gamewidgets) >= 1 and conf.get("closeAll", False):
-                game_handler.closeAllGames(game_handler.gamewidgets)
+            elif len(perspective.gamewidgets) >= 1 and conf.get("closeAll", False):
+                perspective.closeAllGames(perspective.gamewidgets)
                 return True
-        if game_handler.closeAllGames(game_handler.gamewidgets) in (
+        if perspective.closeAllGames(perspective.gamewidgets) in (
                 Gtk.ResponseType.OK, Gtk.ResponseType.YES):
-            Gtk.main_quit()
+            ICLogon.stop()
+            self.app.loop.stop()
+            self.app.quit()
         else:
             return True
 
@@ -244,10 +273,11 @@ class GladeHandlers(object):
 
     def on_rotate_board1_activate(self, widget):
         board_control = newGameDialog.SetupPositionExtension.board_control
+        persp = perspective_manager.get_perspective("games")
         if board_control is not None and board_control.view.is_visible():
             view = newGameDialog.SetupPositionExtension.board_control.view
-        elif gamewidget.cur_gmwidg() is not None:
-            view = gamewidget.cur_gmwidg().board.view
+        elif persp.cur_gmwidg() is not None:
+            view = persp.cur_gmwidg().board.view
         else:
             return
         if view.rotation:
@@ -256,12 +286,12 @@ class GladeHandlers(object):
             view.rotation = math.pi
 
     def on_fullscreen1_activate(self, widget):
-        gamewidget.getWidgets()["window1"].fullscreen()
+        gamewidget.getWidgets()["main_window"].fullscreen()
         gamewidget.getWidgets()["fullscreen1"].hide()
         gamewidget.getWidgets()["leave_fullscreen1"].show()
 
     def on_leave_fullscreen1_activate(self, widget):
-        gamewidget.getWidgets()["window1"].unfullscreen()
+        gamewidget.getWidgets()["main_window"].unfullscreen()
         gamewidget.getWidgets()["leave_fullscreen1"].hide()
         gamewidget.getWidgets()["fullscreen1"].show()
 
@@ -279,10 +309,14 @@ class GladeHandlers(object):
             LogDialog.hide()
 
     def on_show_sidepanels_activate(self, widget):
-        gamewidget.zoomToBoard(not widget.get_active())
+        perspective = perspective_manager.get_perspective("games")
+        perspective.zoomToBoard(not widget.get_active())
 
     def on_hint_mode_activate(self, widget):
-        for gmwidg in game_handler.gamewidgets:
+        perspective = perspective_manager.get_perspective("games")
+        if perspective is None:
+            return
+        for gmwidg in perspective.gamewidgets:
             if gmwidg.isInFront():
                 if widget.get_active():
                     gmwidg.gamemodel.resume_analyzer(HINT)
@@ -290,7 +324,10 @@ class GladeHandlers(object):
                     gmwidg.gamemodel.pause_analyzer(HINT)
 
     def on_spy_mode_activate(self, widget):
-        for gmwidg in game_handler.gamewidgets:
+        perspective = perspective_manager.get_perspective("games")
+        if perspective is None:
+            return
+        for gmwidg in perspective.gamewidgets:
             if gmwidg.isInFront():
                 if widget.get_active():
                     gmwidg.gamemodel.resume_analyzer(SPY)
@@ -300,17 +337,49 @@ class GladeHandlers(object):
     # Edit menu
 
     def on_copy_pgn_activate(self, widget):
-        gamewidget.cur_gmwidg().copy_pgn()
+        persp = perspective_manager.get_perspective("games")
+        persp.cur_gmwidg().copy_pgn()
 
     def on_copy_fen_activate(self, widget):
-        gamewidget.cur_gmwidg().copy_fen()
+        persp = perspective_manager.get_perspective("games")
+        persp.cur_gmwidg().copy_fen()
 
     def on_manage_engines_activate(self, widget):
         enginesDialog.run(gamewidget.getWidgets())
 
+    def on_download_externals_activate(self, widget):
+        externals_dialog = ExternalsDialog()
+        externals_dialog.show()
+
     def on_preferences_activate(self, widget):
         preferencesDialog.run(gamewidget.getWidgets())
 
+    # Database menu
+
+    def on_new_database1_activate(self, widget):
+        perspective = perspective_manager.get_perspective("database")
+        perspective.create_database()
+
+    def on_import_chessfile_activate(self, widget):
+        perspective = perspective_manager.get_perspective("database")
+        perspective.on_import_clicked(widget)
+
+    def on_database_save_as_activate(self, widget):
+        perspective = perspective_manager.get_perspective("database")
+        perspective.on_save_as_clicked(widget)
+
+    def on_import_endgame_nl_activate(self, widget):
+        perspective = perspective_manager.get_perspective("database")
+        perspective.on_import_endgame_nl()
+
+    def on_import_twic_activate(self, widget):
+        perspective = perspective_manager.get_perspective("database")
+        perspective.on_import_twic()
+
+    def on_update_players_activate(self, widget):
+        perspective = perspective_manager.get_perspective("database")
+        perspective.on_update_players()
+
     # Help menu
 
     def on_about_chess1_activate(self, widget):
@@ -323,25 +392,92 @@ class GladeHandlers(object):
         webbrowser.open("https://www.transifex.com/projects/p/pychess/")
 
     def on_TipOfTheDayMenuItem_activate(self, widget):
-        tipOfTheDay.TipOfTheDay.show()
+        tip_of_the_day = TipOfTheDay()
+        tip_of_the_day.show()
 
 
-class PyChess(object):
-    def __init__(self, log_viewer, chess_file):
+class PyChess(Gtk.Application):
+    def __init__(self, log_viewer, purge_recent, chess_file, ics_host, ics_port, loop):
+        Gtk.Application.__init__(self,
+                                 application_id="org.pychess",
+                                 flags=Gio.ApplicationFlags.NON_UNIQUE)
+        self.loop = loop
+
+        if ics_host:
+            ICLogon.host = ics_host
+        if ics_port:
+            ICLogon.port = ics_port
+
+        self.log_viewer = log_viewer
+        self.purge_recent = purge_recent
+        self.chess_file = chess_file
+        self.window = None
+
+    def do_startup(self):
+        Gtk.Application.do_startup(self)
+
+        if self.purge_recent:
+            items = recentManager.get_items()
+            for item in items:
+                uri = item.get_uri()
+                if item.get_application_info("pychess"):
+                    recentManager.remove_item(uri)
+
         self.git_rev = ""
 
-        self.initGlade(log_viewer)
-        self.handleArgs(chess_file)
-        checkversion()
+        self.initGlade(self.log_viewer)
+        self.addPerspectives()
+        self.handleArgs(self.chess_file)
+        # checkversion()
 
         self.loaded_cids = {}
         self.saved_cids = {}
         self.terminated_cids = {}
 
+        log.info("PyChess %s %s git %s" % (VERSION_NAME, VERSION, self.git_rev))
+        log.info("Command line args: '%s'" % self.chess_file)
+        log.info("Platform: %s" % platform.platform())
+        log.info("Python version: %s.%s.%s" % sys.version_info[0:3])
+        log.info("Pyglib version: %s.%s.%s" % GLib.pyglib_version)
+        log.info("Gtk version: %s.%s.%s" % (Gtk.get_major_version(),
+                                            Gtk.get_minor_version(),
+                                            Gtk.get_micro_version()))
+
+    @asyncio.coroutine
+    def print_tasks(self):
+        while True:
+            print(datetime.datetime.now().time())
+            loop = asyncio.get_event_loop()
+            tasks = asyncio.Task.all_tasks(loop)
+            for task in tasks:
+                print(task)
+            print("------------")
+            yield from asyncio.sleep(10)
+
+    def do_activate(self):
+        # asyncio.async(self.print_tasks())
+        self.add_window(self.window)
+        self.window.show_all()
+        gamewidget.getWidgets()["player_rating1"].hide()
+        gamewidget.getWidgets()["leave_fullscreen1"].hide()
+
+        # Externals download dialog
+        if not conf.get("dont_show_externals_at_startup", False):
+            externals_dialog = ExternalsDialog()
+            externals_dialog.show()
+
+        # Tip of the day dialog
+        if conf.get("show_tip_at_startup", False):
+            tip_of_the_day = TipOfTheDay()
+            tip_of_the_day.show()
+
+        dd = DiscovererDialog(discoverer)
+        self.dd_task = asyncio.async(dd.start())
+
     def on_gmwidg_created(self, gamehandler, gmwidg):
         log.debug("GladeHandlers.on_gmwidg_created: starting")
         # Bring playing window to the front
-        gamewidget.getWidgets()["window1"].present()
+        gamewidget.getWidgets()["main_window"].present()
 
         self.loaded_cids[gmwidg.gamemodel] = gmwidg.gamemodel.connect("game_loaded", self.update_recent)
         self.saved_cids[gmwidg.gamemodel] = gmwidg.gamemodel.connect("game_saved", self.update_recent)
@@ -349,6 +485,9 @@ class PyChess(object):
 
         log.debug("GladeHandlers.on_gmwidg_created: returning")
 
+    def on_chessfile_opened(self, persp, chessfile):
+        self.update_recent(None, chessfile.path)
+
     def on_terminated(self, gamemodel):
         if gamemodel.handler_is_connected(self.loaded_cids[gamemodel]):
             gamemodel.disconnect(self.loaded_cids[gamemodel])
@@ -361,49 +500,47 @@ class PyChess(object):
             del self.terminated_cids[gamemodel]
 
     def update_recent(self, gamemodel, uri):
-        if isinstance(uri, basestring):
+        if isinstance(uri, str):
             path = url2pathname(uri)
             recentManager.add_item("file:" + pathname2url(path))
 
     def initGlade(self, log_viewer):
         # Init glade and the 'GladeHandlers'
-        widgets = uistuff.GladeWidgets("PyChess.glade")
-        widgets.getGlade().connect_signals(GladeHandlers())
+        self.widgets = widgets = uistuff.GladeWidgets("PyChess.glade")
+        self.glade_handlers = GladeHandlers(self)
+        widgets.getGlade().connect_signals(self.glade_handlers)
+        self.window = widgets["main_window"]
 
-        new_game_tasker, internet_game_tasker = NewGameTasker(
-        ), InternetGameTasker()
-        tasker.packTaskers(new_game_tasker, internet_game_tasker)
-        widgets["Background"].add(tasker)
+        # new_game_tasker, internet_game_tasker = NewGameTasker(
+        # ), InternetGameTasker()
+        # tasker.packTaskers(new_game_tasker, internet_game_tasker)
+        # widgets["Background"].add(tasker)
 
         # Redirect widgets
         gamewidget.setWidgets(widgets)
 
-        # Main.py still needs a minimum of information
-        game_handler.connect("gmwidg_created", self.on_gmwidg_created)
-
         # The only menuitems that need special initing
         for widget in ("hint_mode", "spy_mode"):
             widgets[widget].set_sensitive(False)
 
-        uistuff.keep(widgets["hint_mode"], "hint_mode", first_value=True)
-        uistuff.keep(widgets["spy_mode"], "spy_mode", first_value=True)
+        uistuff.keep(widgets["hint_mode"], "hint_mode", first_value=False)
+        uistuff.keep(widgets["spy_mode"], "spy_mode", first_value=False)
         uistuff.keep(widgets["show_sidepanels"], "show_sidepanels", first_value=True)
         uistuff.keep(widgets["auto_call_flag"], "autoCallFlag", first_value=True)
 
         # Show main window and init d'n'd
-        widgets["window1"].set_title('%s - PyChess' % _('Welcome'))
-        widgets["window1"].connect("delete-event", GladeHandlers.__dict__["on_quit1_activate"])
-        widgets["window1"].connect("key-press-event", GladeHandlers.__dict__["on_window_key_press"])
+        widgets["main_window"].set_title('%s - PyChess' % _('Welcome'))
+        widgets["main_window"].connect("delete-event", self.glade_handlers.on_quit1_activate)
+        widgets["main_window"].connect("key-press-event", self.glade_handlers.on_window_key_press)
 
-        uistuff.keepWindowSize("main", widgets["window1"], None, POSITION_GOLDEN)
-        widgets["window1"].show()
-        widgets["Background"].show_all()
+        uistuff.keepWindowSize("main", widgets["main_window"], None, uistuff.POSITION_GOLDEN)
 
         # To get drag in the whole window, we add it to the menu and the
         # background. If it can be gotten to work, the drag_dest_set_proxy
         # method is very interesting.
         widgets["menubar1"].drag_dest_set(Gtk.DestDefaults.ALL, DND_LIST, DRAG_ACTION)
-        widgets["Background"].drag_dest_set(Gtk.DestDefaults.ALL, DND_LIST, DRAG_ACTION)
+        widgets["box2"].drag_dest_set(Gtk.DestDefaults.ALL, DND_LIST, DRAG_ACTION)
+        widgets["perspectives_notebook"].drag_dest_set(Gtk.DestDefaults.ALL, DND_LIST, DRAG_ACTION)
 
         # Init 'minor' dialogs
 
@@ -418,7 +555,7 @@ class PyChess(object):
         # About dialog
         self.aboutdialog = widgets["aboutdialog1"]
         self.aboutdialog.set_program_name(NAME)
-        self.aboutdialog.set_copyright("Copyright © 2006-2016")
+        self.aboutdialog.set_copyright("Copyright © 2006-2017")
         self.aboutdialog.set_version(VERSION_NAME + " " + VERSION)
         if os.path.isdir(prefix.addDataPrefix(".git")):
             try:
@@ -447,7 +584,8 @@ class PyChess(object):
             uri = self.get_current_uri()
             try:
                 urlopen(unquote(uri)).close()
-                newGameDialog.LoadFileExtension.run(self.get_current_uri())
+                perspective = perspective_manager.get_perspective("database")
+                perspective.open_chessfile(self.get_current_uri())
             except (IOError, OSError):
                 # shomething wrong whit the uri
                 recentManager.remove_item(uri)
@@ -460,89 +598,47 @@ class PyChess(object):
 
         self.file_filter = Gtk.RecentFilter()
         self.file_filter.add_mime_type("application/x-chess-pgn")
+        self.file_filter.add_mime_type("application/x-chess-epd")
+        self.file_filter.add_mime_type("application/x-chess-fen")
         self.menu_recent.set_filter(self.file_filter)
 
         self.menu_recent.connect("item-activated", recent_item_activated)
         widgets["load_recent_game1"].set_submenu(self.menu_recent)
 
-        # Discoverer dialog
-        def discovering_started(discoverer, binnames):
-            GLib.idle_add(DiscovererDialog.show, discoverer,
-                          widgets["window1"], binnames)
-
-        discoverer.connect("discovering_started", discovering_started)
-        DiscovererDialog.init(discoverer)
-        discoverer.discover()
-
-        # Tip of the day dialog
-        if conf.get("show_tip_at_startup", False):
-            tipOfTheDay.TipOfTheDay.show()
-
         if conf.get("autoLogin", False):
             internet_game_tasker.connectClicked(None)
 
     def website(self, clb, link):
         webbrowser.open(link)
 
+    def addPerspectives(self):
+        perspective_manager.set_widgets(self.widgets)
+        for persp in (Welcome, Games, FICS, Database):
+            perspective = persp()
+            perspective_manager.add_perspective(perspective)
+            perspective.create_toolbuttons()
+
+            if persp == Database:
+                perspective.connect("chessfile_opened", self.on_chessfile_opened)
+            elif persp == Games:
+                perspective.connect("gmwidg_created", self.on_gmwidg_created)
+
+        new_button = Gtk.ToolButton.new_from_stock(Gtk.STOCK_NEW)
+        new_button.set_tooltip_text(_("New Game"))
+        new_button.connect("clicked", self.glade_handlers.on_new_game1_activate)
+        perspective_manager.toolbar.insert(new_button, 0)
+
+        open_button = Gtk.ToolButton.new_from_stock(Gtk.STOCK_OPEN)
+        open_button.set_tooltip_text(_("Open Game"))
+        open_button.connect("clicked", self.glade_handlers.on_load_game1_activate)
+        perspective_manager.toolbar.insert(open_button, 1)
+
     def handleArgs(self, chess_file):
         if chess_file:
 
             def do(discoverer):
-                GLib.idle_add(newGameDialog.LoadFileExtension.run, chess_file)
+                perspective = perspective_manager.get_perspective("database")
+                perspective.open_chessfile(chess_file)
+                self.dd_task.cancel()
 
             discoverer.connect_after("all_engines_discovered", do)
-
-
-def run(no_debug, idle_add_debug, thread_debug, log_viewer, purge_recent,
-        chess_file, ics_host, ics_port):
-    # Start logging
-    if log_viewer:
-        log.logger.addHandler(GLogHandler(logemitter))
-    log.logger.setLevel(logging.WARNING if no_debug is True else logging.DEBUG)
-    oldlogs = [l for l in os.listdir(getUserDataPrefix())
-               if l.endswith(".log")]
-    conf.set("max_log_files", conf.get("max_log_files", 10))
-    oldlogs.sort()
-    l = len(oldlogs)
-    while l > conf.get("max_log_files", 10):
-        try:
-            os.remove(addUserDataPrefix(oldlogs[0]))
-            del oldlogs[0]
-        except OSError:
-            pass
-        l -= 1
-
-    if purge_recent:
-        items = recentManager.get_items()
-        for item in items:
-            uri = item.get_uri()
-            if item.get_application_info("pychess"):
-                recentManager.remove_item(uri)
-
-    signal.signal(signal.SIGINT, Gtk.main_quit)
-    signal.signal(signal.SIGTERM, Gtk.main_quit)
-
-    def cleanup():
-        ICLogon.stop()
-        SubProcess.finishAllSubprocesses()
-
-    atexit.register(cleanup)
-
-    pychess = PyChess(log_viewer, chess_file)
-    idle_add.debug = idle_add_debug
-
-    sys.stdout = LogPipe(sys.stdout, "stdout")
-    sys.stderr = LogPipe(sys.stderr, "stdout")
-    log.info("PyChess %s %s git %s" % (VERSION_NAME, VERSION, pychess.git_rev))
-    log.info("Command line args: '%s'" % chess_file)
-    log.info("Platform: %s" % platform.platform())
-    log.info("Python version: %s.%s.%s" % sys.version_info[0:3])
-    log.info("Pyglib version: %s.%s.%s" % GLib.pyglib_version)
-    if thread_debug:
-        start_thread_dump()
-    if ics_host:
-        ICLogon.host = ics_host
-    if ics_port:
-        ICLogon.port = ics_port
-
-    Gtk.main()
