@@ -1,10 +1,10 @@
-from __future__ import absolute_import
-
 from threading import Thread
+from urllib.parse import urlencode
+from urllib.request import urlopen
+
 from gi.repository import GObject
 
 
-from pychess.compat import urlopen, urlencode
 from pychess.System import fident
 from pychess.System.Log import log
 from pychess.Utils.Offer import Offer
@@ -99,6 +99,10 @@ class Engine(Player):
     # General chat handling
 
     def putMessage(self, message):
+        # TODO: use pandorabots API
+        # https://developer.pandorabots.com/docs
+        return
+
         def answer(message):
             try:
                 data = urlopen(
