@@ -1,11 +1,10 @@
-from __future__ import absolute_import
-from __future__ import print_function
 
+import asyncio
 import collections
 from copy import copy
-from threading import RLock, Thread
 
-from pychess.compat import Queue
+
+from pychess.Utils import wait_signal
 from pychess.Utils.Move import parseAny
 from pychess.Utils.Board import Board
 from pychess.Utils.Move import toAN
@@ -15,11 +14,11 @@ from pychess.Utils.const import CASTLE_KK, ANALYZING, WON_ADJUDICATION, FISCHERR
     WON_MATE, BLACK, BLACKWON, WHITEWON
 from pychess.Utils.lutils.ldata import MATE_VALUE
 from pychess.Utils.lutils.lmove import ParsingError
-from pychess.System import conf, fident
+from pychess.System import conf
 from pychess.System.Log import log
 from pychess.Variants.fischerandom import FischerandomBoard
 
-from .ProtocolEngine import ProtocolEngine
+from .ProtocolEngine import ProtocolEngine, TIME_OUT_SECOND
 from pychess.Players.Player import PlayerIsDead, TurnInterrupt, InvalidMove
 
 TYPEDIC = {"check": lambda x: x == "true", "spin": int}
@@ -37,11 +36,9 @@ class UCIEngine(ProtocolEngine):
         self.wtime = 60000
         self.btime = 60000
         self.incr = 0
+        self.moves = 0
         self.timeHandicap = 1
 
-        self.moveLock = RLock()
-        # none of the following variables should be changed or used in a
-        # condition statement without holding the above self.moveLock
         self.ponderOn = False
         self.pondermove = None
         self.ignoreNext = False
@@ -59,44 +56,41 @@ class UCIEngine(ProtocolEngine):
         self.uciPositionListsMoves = False
         self.analysis = [None]
 
-        self.returnQueue = Queue()
-        self.line_cid = self.engine.connect("line", self.parseLine)
-        self.died_cid = self.engine.connect("died", self.__die)
+        self.queue = asyncio.Queue()
+        self.parse_line_task = asyncio.async(self.parseLine(self.engine))
+        self.died_cid = self.engine.connect("died", lambda e: self.queue.put_nowait("die"))
         self.invalid_move = None
 
         self.cids = [
-            self.connect("readyForOptions", self.__onReadyForOptions_before),
             self.connect_after("readyForOptions", self.__onReadyForOptions),
             self.connect_after("readyForMoves", self.__onReadyForMoves),
         ]
 
-    def __die(self, subprocess):
-        self.returnQueue.put("die")
-
     # Starting the game
 
     def prestart(self):
         print("uci", file=self.engine)
 
-    def start(self):
-        if self.mode in (ANALYZING, INVERSE_ANALYZING):
-            thread = Thread(target=self.__startBlocking,
-                            name=fident(self.__startBlocking))
-            thread.daemon = True
-            thread.start()
-        else:
-            self.__startBlocking()
+    def start(self, event=None):
+        asyncio.async(self.__startBlocking(event))
 
-    def __startBlocking(self):
-        return_queue = self.returnQueue.get()
-        if return_queue == 'die':
+    @asyncio.coroutine
+    def __startBlocking(self, event):
+        try:
+            return_value = yield from asyncio.wait_for(self.queue.get(), TIME_OUT_SECOND)
+        except asyncio.TimeoutError:
+            log.warning("Got timeout error", extra={"task": self.defname})
             raise PlayerIsDead
-        assert return_queue == "ready" or return_queue == 'del'
-        # self.emit("readyForOptions")
-        # self.emit("readyForMoves")
+        except:
+            log.warning("Unknown error", extra={"task": self.defname})
+            raise PlayerIsDead
+        else:
+            if return_value == 'die':
+                raise PlayerIsDead
+            assert return_value == "ready" or return_value == "del"
 
-    def __onReadyForOptions_before(self, self_):
-        self.readyOptions = True
+        if event is not None:
+            event.set()
 
     def __onReadyForOptions(self, self_):
         if self.mode in (ANALYZING, INVERSE_ANALYZING):
@@ -115,20 +109,17 @@ class UCIEngine(ProtocolEngine):
         print("isready", file=self.engine)
 
     def __onReadyForMoves(self, self_):
-        self.returnQueue.put("ready")
         self.readyMoves = True
+        self.queue.put_nowait("ready")
         self._newGame()
 
-        # If we are an analyzer, this signal was already called in a different
-        # thread, so we can safely block it.
         if self.mode in (ANALYZING, INVERSE_ANALYZING):
             self._searchNow()
 
     # Ending the game
 
     def end(self, status, reason):
-        if self.engine.handler_is_connected(self.line_cid):
-            self.engine.disconnect(self.line_cid)
+        self.parse_line_task.cancel()
         if self.engine.handler_is_connected(self.died_cid):
             self.engine.disconnect(self.died_cid)
         if self.handler_is_connected(self.analyze_cid):
@@ -142,7 +133,7 @@ class UCIEngine(ProtocolEngine):
         if self.connected:
             # UCI doens't care about reason, so we just kill
             if reason == WON_ADJUDICATION:
-                self.returnQueue.put("invalid")
+                self.queue.put_nowait("invalid")
             self.kill(reason)
 
     def kill(self, reason):
@@ -156,15 +147,14 @@ class UCIEngine(ProtocolEngine):
                 try:
                     print("stop", file=self.engine)
                     print("quit", file=self.engine)
-                    self.returnQueue.put("del")
-                    return self.engine.gentleKill()
+                    self.queue.put_nowait("del")
+                    self.engine.terminate()
 
                 except OSError as e:
                     # No need to raise on a hang up error, as the engine is dead
                     # anyways
                     if e.errno == 32:
-                        log.warning("Hung up Error",
-                                    extra={"task": self.defname})
+                        log.warning("Hung up Error", extra={"task": self.defname})
                         return e.errno
                     else:
                         raise
@@ -226,44 +216,40 @@ class UCIEngine(ProtocolEngine):
             return
         self._searchNow()
 
+    @asyncio.coroutine
     def makeMove(self, board1, move, board2):
         log.debug("makeMove: move=%s self.pondermove=%s board1=%s board2=%s self.board=%s" % (
             move, self.pondermove, board1, board2, self.board), extra={"task": self.defname})
         assert self.readyMoves
 
-        with self.moveLock:
-            self._recordMove(board1, move, board2)
-            self.waitingForMove = True
-            ponderhit = False
+        self._recordMove(board1, move, board2)
+        self.waitingForMove = True
+        ponderhit = False
 
-            if board2 and self.pondermove and move == self.pondermove:
-                ponderhit = True
-            elif board2 and self.pondermove:
-                self.ignoreNext = True
-                print("stop", file=self.engine)
+        if board2 and self.pondermove and move == self.pondermove:
+            ponderhit = True
+        elif board2 and self.pondermove:
+            self.ignoreNext = True
+            print("stop", file=self.engine)
 
-            self._searchNow(ponderhit=ponderhit)
+        self._searchNow(ponderhit=ponderhit)
 
         # Parse outputs
         try:
-            return_queue = self.returnQueue.get()
+            return_queue = yield from self.queue.get()
             if return_queue == "invalid":
                 raise InvalidMove
             if return_queue == "del":
                 raise PlayerIsDead
             if return_queue == "int":
-                with self.moveLock:
-                    self.pondermove = None
-                    self.ignoreNext = True
-                    self.needBestmove = True
-                    self.hurry()
-                    raise TurnInterrupt
+                self.pondermove = None
+                self.ignoreNext = True
+                self.needBestmove = True
+                self.hurry()
+                raise TurnInterrupt
             return return_queue
         finally:
-            with self.moveLock:
-                self.waitingForMove = False
-                # empty the queue of any moves received post-undo/TurnInterrupt
-                self.returnQueue.queue.clear()
+            self.waitingForMove = False
 
     def updateTime(self, secs, opsecs):
         if self.color == WHITE:
@@ -292,10 +278,11 @@ class UCIEngine(ProtocolEngine):
         elif self.hasOption("UCI_Variant") and not variant.standard_rules:
             self.setOption("UCI_Variant", variant.cecp_name)
 
-    def setOptionTime(self, secs, gain):
+    def setOptionTime(self, secs, gain, moves):
         self.wtime = int(max(secs * 1000 * self.timeHandicap, 1))
         self.btime = int(max(secs * 1000 * self.timeHandicap, 1))
         self.incr = int(gain * 1000 * self.timeHandicap)
+        self.moves = moves
 
     def setOptionStrength(self, strength, forcePonderOff):
         self.strength = strength
@@ -305,8 +292,8 @@ class UCIEngine(ProtocolEngine):
             if self.hasOption('UCI_Elo'):
                 self.setOption('UCI_Elo', 150 * strength)
 
-        # Stockfish offers 20 skill levels
-        if self.hasOption('Skill Level') and strength <= 19:
+        # Stockfish and anticrux engines offer 20 skill levels
+        if self.hasOption('Skill Level'):
             self.setOption('Skill Level', strength)
 
         if ((not self.hasOption('UCI_Elo')) and
@@ -329,34 +316,20 @@ class UCIEngine(ProtocolEngine):
         self.engine.pause()
         return
 
-        if self.board.color == self.color or \
-                self.mode != NORMAL or self.pondermove:
-            self.ignoreNext = True
-            print("stop", file=self.engine)
-
     def resume(self):
         log.debug("resume: self=%s" % self, extra={"task": self.defname})
         self.engine.resume()
         return
 
-        if self.mode == NORMAL:
-            if self.board.color == self.color:
-                self._searchNow()
-            elif self.ponderOn and self.pondermove:
-                self._startPonder()
-        else:
-            self._searchNow()
-
     def hurry(self):
         log.debug("hurry: self.waitingForMove=%s self.readyForStop=%s" % (
             self.waitingForMove, self.readyForStop), extra={"task": self.defname})
         # sending this more than once per move will crash most engines
         # so we need to send only the first one, and then ignore every "hurry" request
         # after that until there is another outstanding "position..go"
-        with self.moveLock:
-            if self.waitingForMove and self.readyForStop:
-                print("stop", file=self.engine)
-                self.readyForStop = False
+        if self.waitingForMove and self.readyForStop:
+            print("stop", file=self.engine)
+            self.readyForStop = False
 
     def playerUndoMoves(self, moves, gamemodel):
         log.debug("playerUndoMoves: moves=%s \
@@ -373,9 +346,9 @@ class UCIEngine(ProtocolEngine):
             # Interrupt if we were searching but should no longer do so, or
             # if it is was our move before undo and it is still our move after undo
             # since we need to send the engine the new FEN in makeMove()
-            log.debug("playerUndoMoves: putting 'int' into self.returnQueue=%s" %
-                      self.returnQueue.queue, extra={"task": self.defname})
-            self.returnQueue.put("int")
+            log.debug("playerUndoMoves: putting 'int' into self.queue=%s" %
+                      self.queue, extra={"task": self.defname})
+            self.queue.put_nowait("int")
 
     def spectatorUndoMoves(self, moves, gamemodel):
         log.debug("spectatorUndoMoves: moves=%s \
@@ -413,17 +386,14 @@ class UCIEngine(ProtocolEngine):
         self.ponderOn = key == "Ponder" and value is True
 
     def getOption(self, option):
-        assert self.readyOptions
         if option in self.options:
             return self.options[option]["default"]
         return None
 
     def getOptions(self):
-        assert self.readyOptions
         return copy(self.options)
 
     def hasOption(self, key):
-        assert self.readyOptions
         return key in self.options
 
     # Internal
@@ -435,57 +405,60 @@ class UCIEngine(ProtocolEngine):
         log.debug("_searchNow: self.needBestmove=%s ponderhit=%s self.board=%s" % (
             self.needBestmove, ponderhit, self.board), extra={"task": self.defname})
 
-        with self.moveLock:
-            commands = []
+        commands = []
 
-            if ponderhit:
-                commands.append("ponderhit")
+        if ponderhit:
+            commands.append("ponderhit")
 
-            elif self.mode == NORMAL:
-                commands.append("position %s" % self.uciPosition)
-                if self.strength <= 3:
-                    commands.append("go depth %d" % self.strength)
+        elif self.mode == NORMAL:
+            commands.append("position %s" % self.uciPosition)
+            if self.strength <= 3:
+                commands.append("go depth %d" % self.strength)
+            else:
+                if self.moves > 0:
+                    commands.append("go wtime %d winc %d btime %d binc %d movestogo %s" % (
+                                    self.wtime, self.incr, self.btime, self.incr, self.moves))
                 else:
                     commands.append("go wtime %d winc %d btime %d binc %d" % (
                                     self.wtime, self.incr, self.btime, self.incr))
 
+        else:
+            print("stop", file=self.engine)
+
+            if self.mode == INVERSE_ANALYZING:
+                if self.board.board.opIsChecked():
+                    # Many engines don't like positions able to take down enemy
+                    # king. Therefore we just return the "kill king" move
+                    # automaticaly
+                    self.emit("analyze", [([toAN(
+                        self.board, getMoveKillingKing(self.board))], MATE_VALUE - 1, "")])
+                    return
+                commands.append("position fen %s" % self.board.asFen())
             else:
-                print("stop", file=self.engine)
+                commands.append("position %s" % self.uciPosition)
 
-                if self.mode == INVERSE_ANALYZING:
-                    if self.board.board.opIsChecked():
-                        # Many engines don't like positions able to take down enemy
-                        # king. Therefore we just return the "kill king" move
-                        # automaticaly
-                        self.emit("analyze", [([toAN(
-                            self.board, getMoveKillingKing(self.board))], MATE_VALUE - 1, "")])
-                        return
-                    commands.append("position fen %s" % self.board.asFen())
-                else:
-                    commands.append("position %s" % self.uciPosition)
+            # commands.append("go infinite")
+            move_time = int(conf.get("max_analysis_spin", 3)) * 1000
+            commands.append("go movetime %s" % move_time)
 
-                # commands.append("go infinite")
-                move_time = int(conf.get("max_analysis_spin", 3)) * 1000
-                commands.append("go movetime %s" % move_time)
+        if self.hasOption("MultiPV") and self.multipvSetting > 1:
+            self.multipvExpected = min(self.multipvSetting,
+                                       legalMoveCount(self.board))
+        else:
+            self.multipvExpected = 1
+        self.analysis = [None] * self.multipvExpected
 
-            if self.hasOption("MultiPV") and self.multipvSetting > 1:
-                self.multipvExpected = min(self.multipvSetting,
-                                           legalMoveCount(self.board))
-            else:
-                self.multipvExpected = 1
-            self.analysis = [None] * self.multipvExpected
-
-            if self.needBestmove:
-                self.commands.append(commands)
-                log.debug("_searchNow: self.needBestmove==True, appended to self.commands=%s" %
-                          self.commands, extra={"task": self.defname})
-            else:
-                for command in commands:
-                    print(command, file=self.engine)
-                if getStatus(self.board)[
-                        1] != WON_MATE:  # XXX This looks fishy.
-                    self.needBestmove = True
-                    self.readyForStop = True
+        if self.needBestmove:
+            self.commands.append(commands)
+            log.debug("_searchNow: self.needBestmove==True, appended to self.commands=%s" %
+                      self.commands, extra={"task": self.defname})
+        else:
+            for command in commands:
+                print(command, file=self.engine)
+            if getStatus(self.board)[
+                    1] != WON_MATE:  # XXX This looks fishy.
+                self.needBestmove = True
+                self.readyForStop = True
 
     def _startPonder(self):
         uciPos = self.uciPosition
@@ -497,179 +470,182 @@ class UCIEngine(ProtocolEngine):
 
     # Parsing from engine
 
-    def parseLine(self, engine, line):
-        if not self.connected:
-            return
-        parts = line.split()
-        if not parts:
-            return
-        # Initializing
-        if parts[0] == "id":
-            if parts[1] == "name":
-                self.ids[parts[1]] = " ".join(parts[2:])
-                self.setName(self.ids["name"])
-            return
+    @asyncio.coroutine
+    def parseLine(self, proc):
+        while True:
+            line = yield from wait_signal(proc, 'line')
+            if not line:
+                break
+            else:
+                line = line[1]
+                parts = line.split()
+                if not parts:
+                    continue
+                # Initializing
+                if parts[0] == "id":
+                    if parts[1] == "name":
+                        self.ids[parts[1]] = " ".join(parts[2:])
+                        self.setName(self.ids["name"])
+                    continue
 
-        if parts[0] == "uciok":
-            self.emit("readyForOptions")
-            return
+                if parts[0] == "uciok":
+                    self.emit("readyForOptions")
+                    continue
 
-        if parts[0] == "readyok":
-            self.emit("readyForMoves")
-            return
+                if parts[0] == "readyok":
+                    self.emit("readyForMoves")
+                    continue
 
-        # Options parsing
-        if parts[0] == "option":
-            dic = {}
-            last = 1
-            varlist = []
-            for i in range(2, len(parts) + 1):
-                if i == len(parts) or parts[i] in OPTKEYS:
-                    key = parts[last]
-                    value = " ".join(parts[last + 1:i])
-                    if "type" in dic and dic["type"] in TYPEDIC:
-                        value = TYPEDIC[dic["type"]](value)
+                # Options parsing
+                if parts[0] == "option":
+                    dic = {}
+                    last = 1
+                    varlist = []
+                    for i in range(2, len(parts) + 1):
+                        if i == len(parts) or parts[i] in OPTKEYS:
+                            key = parts[last]
+                            value = " ".join(parts[last + 1:i])
+                            if "type" in dic and dic["type"] in TYPEDIC:
+                                value = TYPEDIC[dic["type"]](value)
 
-                    if key == "var":
-                        varlist.append(value)
-                    elif key == "type" and value == "string":
-                        dic[key] = "text"
+                            if key == "var":
+                                varlist.append(value)
+                            elif key == "type" and value == "string":
+                                dic[key] = "text"
+                            else:
+                                dic[key] = value
+
+                            last = i
+                    if varlist:
+                        dic["choices"] = varlist
+
+                    if "name" in dic:
+                        self.options[dic["name"]] = dic
+                    continue
+
+                # A Move
+                if self.mode == NORMAL and parts[0] == "bestmove":
+                    self.needBestmove = False
+                    self.__sendQueuedGo()
+
+                    if self.ignoreNext:
+                        log.debug(
+                            "__parseLine: line='%s' self.ignoreNext==True, returning" % line.strip(),
+                            extra={"task": self.defname})
+                        self.ignoreNext = False
+                        self.readyForStop = True
+                        continue
+
+                    movestr = parts[1]
+                    if not self.waitingForMove:
+                        log.warning("__parseLine: self.waitingForMove==False, ignoring move=%s" % movestr,
+                                    extra={"task": self.defname})
+                        self.pondermove = None
+                        continue
+                    self.waitingForMove = False
+
+                    try:
+                        move = parseAny(self.board, movestr)
+                    except ParsingError:
+                        self.invalid_move = movestr
+                        self.end(WHITEWON if self.board.color == BLACK else
+                                 BLACKWON, WON_ADJUDICATION)
+                        continue
+
+                    if not validate(self.board, move):
+                        # This is critical. To avoid game stalls, we need to resign on
+                        # behalf of the engine.
+                        log.error("__parseLine: move=%s didn't validate, putting 'del' \
+                                  in returnQueue. self.board=%s" % (
+                            repr(move), self.board), extra={"task": self.defname})
+                        self.invalid_move = movestr
+                        self.end(WHITEWON if self.board.color == BLACK else
+                                 BLACKWON, WON_ADJUDICATION)
+                        continue
+
+                    self._recordMove(self.board.move(move), move, self.board)
+                    log.debug("__parseLine: applied move=%s to self.board=%s" % (
+                        move, self.board), extra={"task": self.defname})
+
+                    if self.ponderOn:
+                        self.pondermove = None
+                        # An engine may send an empty ponder line, simply to clear.
+                        if len(parts) == 4:
+                            # Engines don't always check for everything in their
+                            # ponders. Hence we need to validate.
+                            # But in some cases, what they send may not even be
+                            # correct AN - specially in the case of promotion.
+                            try:
+                                pondermove = parseAny(self.board, parts[3])
+                            except ParsingError:
+                                pass
+                            else:
+                                if validate(self.board, pondermove):
+                                    self.pondermove = pondermove
+                                    self._startPonder()
+
+                    self.queue.put_nowait(move)
+                    log.debug("__parseLine: put move=%s into self.queue=%s" % (
+                        move, self.queue), extra={"task": self.defname})
+                    continue
+
+                # An Analysis
+                if self.mode != NORMAL and parts[0] == "info" and "pv" in parts:
+                    multipv = 1
+                    if "multipv" in parts:
+                        multipv = int(parts[parts.index("multipv") + 1])
+                    scoretype = parts[parts.index("score") + 1]
+                    if scoretype in ('lowerbound', 'upperbound'):
+                        score = None
                     else:
-                        dic[key] = value
+                        score = int(parts[parts.index("score") + 2])
+                        if scoretype == 'mate':
+                            #                    print >> self.engine, "stop"
+                            if score != 0:
+                                sign = score / abs(score)
+                                score = sign * MATE_VALUE
 
-                    last = i
-            if varlist:
-                dic["choices"] = varlist
+                    movstrs = parts[parts.index("pv") + 1:]
 
-            self.options[dic["name"]] = dic
-            return
+                    if "depth" in parts:
+                        depth = parts[parts.index("depth") + 1]
+                    else:
+                        depth = ""
 
-        # A Move
-        if self.mode == NORMAL and parts[0] == "bestmove":
-            with self.moveLock:
-                self.needBestmove = False
-                self.__sendQueuedGo()
+                    if multipv <= len(self.analysis):
+                        self.analysis[multipv - 1] = (movstrs, score, depth)
 
-                if self.ignoreNext:
-                    log.debug(
-                        "__parseLine: line='%s' self.ignoreNext==True, returning" % line.strip(),
-                        extra={"task": self.defname})
+                    self.emit("analyze", self.analysis)
+                    continue
+
+                # An Analyzer bestmove
+                if self.mode != NORMAL and parts[0] == "bestmove":
+                    log.debug("__parseLine: processing analyzer bestmove='%s'" % line.strip(),
+                              extra={"task": self.defname})
+                    self.needBestmove = False
+                    self.__sendQueuedGo(sendlast=True)
+                    continue
+
+                # Stockfish complaining it received a 'stop' without a corresponding 'position..go'
+                if line.strip() == "Unknown command: stop":
+                    log.debug("__parseLine: processing '%s'" % line.strip(),
+                              extra={"task": self.defname})
                     self.ignoreNext = False
-                    self.readyForStop = True
-                    return
+                    self.needBestmove = False
+                    self.readyForStop = False
+                    self.__sendQueuedGo()
+                    continue
 
-                movestr = parts[1]
-                if not self.waitingForMove:
-                    log.warning("__parseLine: self.waitingForMove==False, ignoring move=%s" % movestr,
-                                extra={"task": self.defname})
-                    self.pondermove = None
-                    return
-                self.waitingForMove = False
-
-                try:
-                    move = parseAny(self.board, movestr)
-                except ParsingError:
-                    self.invalid_move = movestr
-                    self.end(WHITEWON if self.board.color == BLACK else
-                             BLACKWON, WON_ADJUDICATION)
-                    return
-
-                if not validate(self.board, move):
-                    # This is critical. To avoid game stalls, we need to resign on
-                    # behalf of the engine.
-                    log.error("__parseLine: move=%s didn't validate, putting 'del' \
-                              in returnQueue. self.board=%s" % (
-                        repr(move), self.board), extra={"task": self.defname})
-                    self.invalid_move = movestr
-                    self.end(WHITEWON if self.board.color == BLACK else
-                             BLACKWON, WON_ADJUDICATION)
-                    return
-
-                self._recordMove(self.board.move(move), move, self.board)
-                log.debug("__parseLine: applied move=%s to self.board=%s" % (
-                    move, self.board), extra={"task": self.defname})
-
-                if self.ponderOn:
-                    self.pondermove = None
-                    # An engine may send an empty ponder line, simply to clear.
-                    if len(parts) == 4:
-                        # Engines don't always check for everything in their
-                        # ponders. Hence we need to validate.
-                        # But in some cases, what they send may not even be
-                        # correct AN - specially in the case of promotion.
-                        try:
-                            pondermove = parseAny(self.board, parts[3])
-                        except ParsingError:
-                            pass
-                        else:
-                            if validate(self.board, pondermove):
-                                self.pondermove = pondermove
-                                self._startPonder()
-
-                self.returnQueue.put(move)
-                log.debug("__parseLine: put move=%s into self.returnQueue=%s" % (
-                    move, self.returnQueue.queue), extra={"task": self.defname})
-                return
-
-        # An Analysis
-        if self.mode != NORMAL and parts[0] == "info" and "pv" in parts:
-            multipv = 1
-            if "multipv" in parts:
-                multipv = int(parts[parts.index("multipv") + 1])
-            scoretype = parts[parts.index("score") + 1]
-            if scoretype in ('lowerbound', 'upperbound'):
-                score = None
-            else:
-                score = int(parts[parts.index("score") + 2])
-                if scoretype == 'mate':
-                    #                    print >> self.engine, "stop"
-                    if score != 0:
-                        sign = score / abs(score)
-                        score = sign * MATE_VALUE
-
-            movstrs = parts[parts.index("pv") + 1:]
-
-            if "depth" in parts:
-                depth = parts[parts.index("depth") + 1]
-            else:
-                depth = ""
-
-            if multipv <= len(self.analysis):
-                self.analysis[multipv - 1] = (movstrs, score, depth)
-
-            self.emit("analyze", self.analysis)
-            return
-
-        # An Analyzer bestmove
-        if self.mode != NORMAL and parts[0] == "bestmove":
-            with self.moveLock:
-                log.debug("__parseLine: processing analyzer bestmove='%s'" % line.strip(),
-                          extra={"task": self.defname})
-                self.needBestmove = False
-                self.__sendQueuedGo(sendlast=True)
-                return
-
-        # Stockfish complaining it received a 'stop' without a corresponding 'position..go'
-        if line.strip() == "Unknown command: stop":
-            with self.moveLock:
-                log.debug("__parseLine: processing '%s'" % line.strip(),
-                          extra={"task": self.defname})
-                self.ignoreNext = False
-                self.needBestmove = False
-                self.readyForStop = False
-                self.__sendQueuedGo()
-                return
-
-        # * score
-        # * cp <x>
-        #    the score from the engine's point of view in centipawns.
-        # * mate <y>
-        #    mate in y moves, not plies.
-        #    If the engine is getting mated use negative values for y.
-        # * lowerbound
-        #  the score is just a lower bound.
-        # * upperbound
-        #   the score is just an upper bound.
+                # * score
+                # * cp <x>
+                #    the score from the engine's point of view in centipawns.
+                # * mate <y>
+                #    mate in y moves, not plies.
+                #    If the engine is getting mated use negative values for y.
+                # * lowerbound
+                #  the score is just a lower bound.
+                # * upperbound
+                #   the score is just an upper bound.
 
     def __sendQueuedGo(self, sendlast=False):
         """ Sends the next position...go or ponderhit command set which was queued (if any).
@@ -677,20 +653,19 @@ class UCIEngine(ProtocolEngine):
         sendlast -- If True, send the last position-go queued rather than the first,
         and discard the others (intended for analyzers)
         """
-        with self.moveLock:
-            if len(self.commands) > 0:
-                if sendlast:
-                    commands = self.commands.pop()
-                    self.commands.clear()
-                else:
-                    commands = self.commands.popleft()
+        if len(self.commands) > 0:
+            if sendlast:
+                commands = self.commands.pop()
+                self.commands.clear()
+            else:
+                commands = self.commands.popleft()
 
-                for command in commands:
-                    print(command, file=self.engine)
-                self.needBestmove = True
-                self.readyForStop = True
-                log.debug("__sendQueuedGo: sent queued go=%s" % commands,
-                          extra={"task": self.defname})
+            for command in commands:
+                print(command, file=self.engine)
+            self.needBestmove = True
+            self.readyForStop = True
+            log.debug("__sendQueuedGo: sent queued go=%s" % commands,
+                      extra={"task": self.defname})
 
     #    Info
 
@@ -706,11 +681,10 @@ class UCIEngine(ProtocolEngine):
 
         if n != self.multipvSetting:
             conf.set("multipv", n)
-            with self.moveLock:
-                self.multipvSetting = n
-                print("stop", file=self.engine)
-                print("setoption name MultiPV value", n, file=self.engine)
-                self._searchNow()
+            self.multipvSetting = n
+            print("stop", file=self.engine)
+            print("setoption name MultiPV value", n, file=self.engine)
+            self._searchNow()
 
         return n
 
