@@ -1,11 +1,8 @@
-from __future__ import absolute_import
-
-from pychess.compat import PY3
 from pychess.Utils.const import EMPTY, PAWN, KNIGHT, BISHOP, ROOK, QUEEN, KING, \
     ATOMICCHESS, BUGHOUSECHESS, CRAZYHOUSECHESS, CAMBODIANCHESS, MAKRUKCHESS, \
     FISCHERRANDOMCHESS, SITTUYINCHESS, WILDCASTLECHESS, WILDCASTLESHUFFLECHESS, \
     SUICIDECHESS, DROP_VARIANTS, BLACK, WHITE, FAN_PIECES, NULL_MOVE, CAS_FLAGS, \
-    NORMALCHESS, \
+    NORMALCHESS, FEN_START, \
     chrU2Sign, cordDic, reprCord, reprFile, reprSign, reprSignMakruk, reprSignSittuyin, \
     A1, A8, B1, B8, \
     C1, C8, D1, D8, \
@@ -17,7 +14,7 @@ from .ldata import FILE, fileBits
 from .attack import isAttacked
 from .bitboard import clearBit, setBit, bitPosArray
 from .PolyglotHash import pieceHashes, epHashes, \
-    W_OOHash, W_OOOHash, B_OOHash, B_OOOHash, colorHash
+    W_OOHash, W_OOOHash, B_OOHash, B_OOOHash, colorHash, holdingHash
 
 ################################################################################
 # FEN                                                                          #
@@ -74,6 +71,7 @@ class LBoard(object):
         # null_board act as parent of the variation
         # when we add a variation to last played board from hint panel
         self.fen_was_applied = False
+        self.plyCount = 0
 
     @property
     def lastMove(self):
@@ -127,9 +125,9 @@ class LBoard(object):
         # Set board to empty on Black's turn (which Polyglot-hashes to 0)
         self.blocker = 0
 
-        self.friends = [0] * 2
-        self.kings = [-1] * 2
-        self.boards = [[0] * 7 for i in range(2)]
+        self.friends = [0, 0]
+        self.kings = [-1, -1]
+        self.boards = ([0] * 7, [0] * 7)
 
         self.enpassant = None  # cord which can be captured by enpassant or None
         self.color = BLACK
@@ -148,8 +146,8 @@ class LBoard(object):
 
         #  Data from the position's history:
         self.hist_move = []  # The move that was applied to get the position
-        self.hist_tpiece = [
-        ]  # The piece the move captured, == EMPTY for normal moves
+        self.hist_tpiece = []
+        # The piece the move captured, == EMPTY for normal moves
         self.hist_enpassant = []
         self.hist_castling = []
         self.hist_hash = []
@@ -158,7 +156,7 @@ class LBoard(object):
         self.hist_opchecked = []
 
         # piece counts
-        self.pieceCount = [[0] * 7, [0] * 7]
+        self.pieceCount = ([0] * 7, [0] * 7)
 
         # initial cords of rooks and kings for castling in Chess960
         if self.variant == FISCHERRANDOMCHESS:
@@ -249,6 +247,7 @@ class LBoard(object):
                         color = char.islower() and BLACK or WHITE
                         piece = chrU2Sign[char.upper()]
                         self.holding[color][piece] += 1
+                        self.hash ^= holdingHash[color][piece][self.holding[color][piece]]
                         continue
                     else:
                         break
@@ -421,6 +420,11 @@ class LBoard(object):
         board_clone.applyMove(move)
         return board_clone.opIsChecked()
 
+    def willGiveCheck(self, move):
+        board_clone = self.clone()
+        board_clone.applyMove(move)
+        return board_clone.isChecked()
+
     def _addPiece(self, cord, piece, color):
         _setBit = setBit
         self.boards[color][piece] = _setBit(self.boards[color][piece], cord)
@@ -551,10 +555,12 @@ class LBoard(object):
                 if self.promoted[tcord]:
                     if self.variant == CRAZYHOUSECHESS:
                         self.holding[color][PAWN] += 1
+                        self.hash ^= holdingHash[color][PAWN][self.holding[color][PAWN]]
                     self.capture_promoting = True
                 else:
                     if self.variant == CRAZYHOUSECHESS:
                         self.holding[color][tpiece] += 1
+                        self.hash ^= holdingHash[color][tpiece][self.holding[color][tpiece]]
                     self.capture_promoting = False
             elif self.variant == ATOMICCHESS:
                 from pychess.Variants.atomic import piecesAround
@@ -570,7 +576,6 @@ class LBoard(object):
                         elif acord == self.ini_rooks[opcolor][1]:
                             castling &= ~CAS_FLAGS[opcolor][1]
                 self.hist_exploding_around.append(apieces)
-
         self.hist_tpiece.append(tpiece)
 
         # Remove moving piece(s), then add them at their destination.
@@ -578,6 +583,7 @@ class LBoard(object):
             if self.variant in DROP_VARIANTS:
                 assert self.holding[color][fpiece] > 0
             self.holding[color][fpiece] -= 1
+            self.hash ^= holdingHash[color][fpiece][self.holding[color][fpiece]]
             self.pieceCount[color][fpiece] += 1
         else:
             self._removePiece(fcord, fpiece, color)
@@ -593,6 +599,7 @@ class LBoard(object):
             self.pieceCount[opcolor][PAWN] -= 1
             if self.variant == CRAZYHOUSECHESS:
                 self.holding[color][PAWN] += 1
+                self.hash ^= holdingHash[color][PAWN][self.holding[color][PAWN]]
             elif self.variant == ATOMICCHESS:
                 from pychess.Variants.atomic import piecesAround
                 apieces = [(fcord, fpiece, color), ]
@@ -708,9 +715,11 @@ class LBoard(object):
                 if self.capture_promoting:
                     assert self.holding[color][PAWN] > 0
                     self.holding[color][PAWN] -= 1
+                    self.hash ^= holdingHash[color][PAWN][self.holding[color][PAWN]]
                 else:
                     assert self.holding[color][cpiece] > 0
                     self.holding[color][cpiece] -= 1
+                    self.hash ^= holdingHash[color][cpiece][self.holding[color][cpiece]]
             elif self.variant == ATOMICCHESS:
                 apieces = self.hist_exploding_around.pop()
                 for acord, apiece, acolor in apieces:
@@ -725,6 +734,7 @@ class LBoard(object):
             if self.variant == CRAZYHOUSECHESS:
                 assert self.holding[color][PAWN] > 0
                 self.holding[color][PAWN] -= 1
+                self.hash ^= holdingHash[color][PAWN][self.holding[color][PAWN]]
             elif self.variant == ATOMICCHESS:
                 apieces = self.hist_exploding_around.pop()
                 for acord, apiece, acolor in apieces:
@@ -740,6 +750,7 @@ class LBoard(object):
         # Put back moved piece
         if flag == DROP:
             self.holding[color][tpiece] += 1
+            self.hash ^= holdingHash[color][tpiece][self.holding[color][tpiece]]
             self.pieceCount[color][tpiece] -= 1
         else:
             if not (self.variant == ATOMICCHESS and
@@ -838,8 +849,7 @@ class LBoard(object):
         return b
 
     def __repr__(self):
-        b = self.prepr()
-        return b if PY3 else b.encode('utf8')
+        return self.prepr()
 
     def asFen(self, enable_bfen=True):
         fenstr = []
@@ -936,10 +946,9 @@ class LBoard(object):
 
         copy.friends = self.friends[:]
         copy.kings = self.kings[:]
-        copy.boards = [self.boards[WHITE][:], self.boards[BLACK][:]]
+        copy.boards = (self.boards[WHITE][:], self.boards[BLACK][:])
         copy.arBoard = self.arBoard[:]
-        copy.pieceCount = [self.pieceCount[WHITE][:],
-                           self.pieceCount[BLACK][:]]
+        copy.pieceCount = (self.pieceCount[WHITE][:], self.pieceCount[BLACK][:])
 
         copy.color = self.color
         copy.plyCount = self.plyCount
@@ -975,8 +984,7 @@ class LBoard(object):
             copy.capture_promoting = self.capture_promoting
             copy.hist_capture_promoting = self.hist_capture_promoting[:]
         elif self.variant == ATOMICCHESS:
-            copy.hist_exploding_around = [a[:]
-                                          for a in self.hist_exploding_around]
+            copy.hist_exploding_around = [a[:] for a in self.hist_exploding_around]
         elif self.variant == CAMBODIANCHESS:
             copy.ini_kings = self.ini_kings
             copy.ini_queens = self.ini_queens
@@ -986,3 +994,6 @@ class LBoard(object):
 
         copy.fen_was_applied = self.fen_was_applied
         return copy
+
+START_BOARD = LBoard()
+START_BOARD.applyFen(FEN_START)
