@@ -1,10 +1,11 @@
-from __future__ import absolute_import
+import builtins
+
 from .const import ADJOURNED_LOST_CONNECTION_BLACK, UNKNOWN_REASON, WON_LESSMATERIAL, \
     KILLED, ADJOURNED_SERVER_SHUTDOWN, DRAW_EQUALMATERIAL, \
     ABORTED_ADJUDICATION, DISCONNECTED, BLACKWON, WHITEWON, DRAW_CALLFLAG, WON_RESIGN, DRAW, \
-    WON_CALLFLAG, DRAW_WHITEINSUFFICIENTANDBLACKTIME, DRAW_LENGTH, WON_MATE, \
+    WON_CALLFLAG, DRAW_WHITEINSUFFICIENTANDBLACKTIME, DRAW_LENGTH, WON_MATE, WON_KINGINEIGHTROW, \
     DRAW_BLACKINSUFFICIENTANDWHITETIME, ADJOURNED_COURTESY_WHITE, WON_THREECHECK, \
-    DRAW_AGREE, DRAW_INSUFFICIENT, DRAW_STALEMATE, DRAW_ADJUDICATION, \
+    DRAW_AGREE, DRAW_INSUFFICIENT, DRAW_STALEMATE, DRAW_ADJUDICATION, UNKNOWN_STATE, \
     ADJOURNED_LOST_CONNECTION, WON_NOMATERIAL, ADJOURNED, WON_DISCONNECTION, ABORTED, \
     ADJOURNED_AGREEMENT, WON_KINGEXPLODE, WON_KINGINCENTER, ABORTED_SERVER_SHUTDOWN, \
     ABORTED_AGREEMENT, WHITE_ENGINE_DIED, BLACK_ENGINE_DIED, WON_ADJUDICATION, \
@@ -12,8 +13,6 @@ from .const import ADJOURNED_LOST_CONNECTION_BLACK, UNKNOWN_REASON, WON_LESSMATE
     DRAW_50MOVES, ADJOURNED_LOST_CONNECTION_WHITE, ABORTED_EARLY, ABORTED_DISCONNECTION
 
 
-from pychess.compat import builtins
-
 if '_' not in builtins.__dict__:
     builtins.__dict__['_'] = lambda s: s
     builtins.__dict__[
@@ -33,6 +32,7 @@ reprResult_long = {
     KILLED: _("The game has been killed"),
     ADJOURNED: _("The game has been adjourned"),
     ABORTED: _("The game has been aborted"),
+    UNKNOWN_STATE: _("Unknown game state"),
 }
 
 reprReason_long = {
@@ -62,6 +62,7 @@ reprReason_long = {
     WON_KINGEXPLODE: _("Because %(loser)s king exploded"),
     WON_KINGINCENTER: _("Because %(winner)s king reached the center"),
     WON_THREECHECK: _("Because %(winner)s was giving check 3 times"),
+    WON_KINGINEIGHTROW: _("Because %(winner)s king reached the eight row"),
     ADJOURNED_LOST_CONNECTION: _("Because a player lost connection"),
     ADJOURNED_AGREEMENT: _("Because both players agreed to an adjournment"),
     ADJOURNED_SERVER_SHUTDOWN: _("Because the server was shut down"),
