@@ -1,19 +1,18 @@
-from __future__ import absolute_import, print_function
-
+import asyncio
 import re
+import sys
 import socket
-import time
-import threading
-from threading import Event, Thread
+import traceback
 from collections import defaultdict
+from concurrent.futures import CancelledError
 
 from gi.repository import GObject
 
 import pychess
-from pychess.System import fident
 from pychess.System.Log import log
 
-from pychess.ic import NAMES_RE, TITLES_RE
+from pychess import ic
+from pychess.Utils.const import NAME
 from .managers.SeekManager import SeekManager
 from .managers.FingerManager import FingerManager
 from .managers.NewsManager import NewsManager
@@ -26,10 +25,20 @@ from .managers.ListAndVarManager import ListAndVarManager
 from .managers.AutoLogOutManager import AutoLogOutManager
 from .managers.ErrorManager import ErrorManager
 from .managers.AdjournManager import AdjournManager
-from pychess.Utils.const import NAME
+from .managers.ICCSeekManager import ICCSeekManager
+from .managers.ICCBoardManager import ICCBoardManager
+from .managers.ICCChatManager import ICCChatManager
+from .managers.ICCHelperManager import ICCHelperManager
+from .managers.ICCAdjournManager import ICCAdjournManager
+from .managers.ICCErrorManager import ICCErrorManager
+from .managers.ICCFingerManager import ICCFingerManager
+from .managers.ICCListAndVarManager import ICCListAndVarManager
+from .managers.ICCNewsManager import ICCNewsManager
+from .managers.ICCOfferManager import ICCOfferManager
+from .managers.ICCAutoLogOutManager import ICCAutoLogOutManager
 
 from .FICSObjects import FICSPlayers, FICSGames, FICSSeeks, FICSChallenges
-from .TimeSeal import TimeSeal, CanceledException
+from .TimeSeal import CanceledException, ICSTelnet
 from .VerboseTelnet import LinePrediction, FromPlusPrediction, FromABPlusPrediction, \
     FromToPrediction, PredictionsTelnet, NLinesPrediction
 
@@ -38,7 +47,7 @@ class LogOnException(Exception):
     pass
 
 
-class Connection(GObject.GObject, Thread):
+class Connection(GObject.GObject):
 
     __gsignals__ = {
         'connecting': (GObject.SignalFlags.RUN_FIRST, None, ()),
@@ -50,7 +59,6 @@ class Connection(GObject.GObject, Thread):
 
     def __init__(self, host, ports, username, password):
         GObject.GObject.__init__(self)
-        Thread.__init__(self, name=fident(self.run))
         self.daemon = True
         self.host = host
         self.ports = ports
@@ -59,6 +67,7 @@ class Connection(GObject.GObject, Thread):
 
         self.connected = False
         self.connecting = False
+        self.keep_alive_task = None
 
         self.predictions = set()
         self.predictionsDict = {}
@@ -69,12 +78,18 @@ class Connection(GObject.GObject, Thread):
 
         self.USCN = False
 
+        self.ICC = False
+        self.replay_dg_dict = {}
+        self.replay_cn_dict = {}
+
     @property
     def ics_name(self):
         if self.FatICS:
             return "FatICS"
         elif self.USCN:
             return "USCN"
+        elif self.ICC:
+            return "ICC"
         else:
             return "FICS"
 
@@ -101,6 +116,12 @@ class Connection(GObject.GObject, Thread):
             if len(self.reply_cmd_dict[callback.BLKCMD]) == 0:
                 del self.reply_cmd_dict[callback.BLKCMD]
 
+    def expect_dg_line(self, number, callback):
+        self.replay_dg_dict[number] = callback
+
+    def expect_cn_line(self, number, callback):
+        self.replay_cn_dict[number] = callback
+
     def expect_line(self, callback, regexp):
         self.expect(LinePrediction(callback, regexp))
 
@@ -158,18 +179,21 @@ class FICSConnection(Connection):
     def _start_managers(self):
         pass
 
+    @asyncio.coroutine
     def _connect(self):
         self.connecting = True
         self.emit("connecting")
         try:
-            self.client = TimeSeal()
 
             self.emit('connectingMsg', _("Connecting to server"))
             for i, port in enumerate(self.ports):
                 log.debug("Trying port %d" % port,
                           extra={"task": (self.host, "raw")})
                 try:
-                    self.client.open(self.host, port)
+                    connected_event = asyncio.Event()
+                    self.client = ICSTelnet()
+                    asyncio.async(self.client.start(self.host, port, connected_event))
+                    yield from connected_event.wait()
                 except socket.error as err:
                     log.debug("Failed to open port %d %s" % (port, err),
                               extra={"task": (self.host, "raw")})
@@ -180,17 +204,17 @@ class FICSConnection(Connection):
                 else:
                     break
 
-            self.client.read_until("login: ")
+            yield from self.client.read_until("login: ")
             self.emit('connectingMsg', _("Logging on to server"))
 
             # login with registered handle
             if self.password:
-                print(self.username, file=self.client)
-                got = self.client.read_until(
+                self.client.write(self.username)
+                got = yield from self.client.read_until(
                     "password:", "enter the server as", "Try again.")
                 if got == 0:
                     self.client.sensitive = True
-                    print(self.password, file=self.client)
+                    self.client.write(self.password)
                 # No such name
                 elif got == 1:
                     raise LogOnException(NOTREG % self.username)
@@ -199,23 +223,24 @@ class FICSConnection(Connection):
                     raise LogOnException(NOTREG % self.username)
             else:
                 if self.username:
-                    print(self.username, file=self.client)
+                    self.client.write(self.username)
                 else:
-                    print("guest", file=self.client)
-                got = self.client.read_until(
+                    self.client.write("guest")
+                got = yield from self.client.read_until(
                     "Press return",
+                    "You are connected as a guest",
                     "If it is yours, type the password.",
                     "guest connections have been prevented",
                     "nobody from your site may login without an account.")
                 # got = 3
-                if got == 1:
+                if got == 2:
                     raise LogOnException(REGISTERED % self.username)
-                elif got == 2 or got == 3:
+                elif got == 3 or got == 4:
                     raise LogOnException(PREVENTED)
-                print(file=self.client)
+                self.client.write("")
 
             while True:
-                line = self.client.readline()
+                line = yield from self.client.readline()
                 if "Invalid password" in line:
                     raise LogOnException(BADPAS)
                 elif "is already logged in" in line:
@@ -223,37 +248,54 @@ class FICSConnection(Connection):
 
                 match = re.search(
                     "\*\*\*\* Starting FICS session as " + "(%s)%s \*\*\*\*" %
-                    (NAMES_RE, TITLES_RE), line)
+                    (ic.NAMES_RE, ic.TITLES_RE), line)
                 if match:
                     self.username = match.groups()[0]
                     break
 
-                    # USCN specific lines
-                match = re.search("Created temporary login '(%s)'" % NAMES_RE,
-                                  line)
+                # USCN specific line
+                match = re.search("Created temporary login '(%s)'" % ic.NAMES_RE, line)
                 if match:
                     self.username = match.groups()[0]
                     break
-                match = re.search("answers to frequently asked questions",
-                                  line)
+
+                match = re.search("For a list of events, click here:", line)
                 if match:
                     break
+
+                # ICC specific line
+                match = re.search("help anonymous", line)
+                if match:
+                    break
+
                 match = re.search("This is the admin message of the day", line)
                 if match:
                     break
 
             self.emit('connectingMsg', _("Setting up environment"))
-            lines = self.client.readuntil(b"ics%")
+            lines = yield from self.client.readuntil(b"ics%")
             self._post_connect_hook(lines)
             self.FatICS = self.client.FatICS
             self.USCN = self.client.USCN
+            self.ICC = self.client.ICC
             self.client.name = self.username
-            self.client = PredictionsTelnet(self.client, self.predictions,
-                                            self.reply_cmd_dict)
-            self.client.lines.line_prefix = "fics%"
-            if not self.USCN:
+            self.client = PredictionsTelnet(self.client, self.predictions, self.reply_cmd_dict,
+                                            self.replay_dg_dict, self.replay_cn_dict)
+            self.client.lines.line_prefix = "aics%" if self.ICC else "fics%"
+
+            if not self.USCN and not self.ICC:
                 self.client.run_command("iset block 1")
                 self.client.lines.block_mode = True
+
+            if self.ICC:
+                self.client.run_command("set level1 5")
+                self.client.run_command("set prompt 0")
+                self.client.lines.datagram_mode = True
+
+                ic.GAME_TYPES_BY_SHORT_FICS_NAME["B"] = ic.GAME_TYPES["bullet"]
+            else:
+                ic.GAME_TYPES_BY_SHORT_FICS_NAME["B"] = ic.GAME_TYPES["bughouse"]
+
             self.client.run_command("iset defprompt 1")
             self.client.run_command("iset ms 1")
 
@@ -262,17 +304,12 @@ class FICSConnection(Connection):
             self.connected = True
             self.emit("connected")
 
+            @asyncio.coroutine
             def keep_alive():
-                last = time.time()
                 while self.isConnected():
-                    if time.time() - last > 59 * 60:
-                        self.client.run_command("date")
-                        last = time.time()
-                    time.sleep(30)
-
-            thread = threading.Thread(target=keep_alive, name=fident(keep_alive))
-            thread.daemon = True
-            thread.start()
+                    self.client.run_command("date")
+                    yield from asyncio.sleep(30 * 60)
+            self.keep_alive_task = asyncio.async(keep_alive())
 
         except CanceledException as err:
             log.info("FICSConnection._connect: %s" % repr(err),
@@ -280,25 +317,30 @@ class FICSConnection(Connection):
         finally:
             self.connecting = False
 
-    def run(self):
+    @asyncio.coroutine
+    def start(self):
         try:
-            try:
-                if not self.isConnected():
-                    self._connect()
-                while self.isConnected():
-                    self.client.parse()
-            except Exception as err:
-                log.info("FICSConnection.run: %s" % repr(err),
-                         extra={"task": (self.host, "raw")})
-                self.close()
-                if isinstance(err,
-                              (IOError, LogOnException, EOFError, socket.error,
-                               socket.gaierror, socket.herror)):
-                    self.emit("error", err)
-                else:
-                    raise
+            if not self.isConnected():
+                yield from self._connect()
+            while self.isConnected():
+                yield from self.client.parse()
+        except CancelledError:
+            pass
+        except Exception as err:
+            exc_type, exc_value, exc_traceback = sys.exc_info()
+            traceback.print_exception(exc_type, exc_value, exc_traceback)
+            log.info("FICSConnection.run: %s" % repr(err),
+                     extra={"task": (self.host, "raw")})
+            self.close()
+            if isinstance(err,
+                          (IOError, LogOnException, EOFError, socket.error,
+                           socket.gaierror, socket.herror)):
+                self.emit("error", err)
+            else:
+                raise
         finally:
-            self.emit("disconnected")
+            if isinstance(self, FICSMainConnection):
+                self.emit("disconnected")
 
     def cancel(self):
         self.close()
@@ -307,6 +349,8 @@ class FICSConnection(Connection):
     def close(self):
         self.connected = False
         self.client.close()
+        if self.keep_alive_task is not None:
+            self.keep_alive_task.cancel()
 
 
 class FICSMainConnection(FICSConnection):
@@ -315,7 +359,6 @@ class FICSMainConnection(FICSConnection):
         self.lvm = None
         self.notify_users = []
         self.ini_messages = []
-        self.lounge_loaded = Event()
         self.players = FICSPlayers(self)
         self.games = FICSGames(self)
         self.seeks = FICSSeeks(self)
@@ -346,9 +389,9 @@ class FICSMainConnection(FICSConnection):
     def _post_connect_hook(self, lines):
         self.ini_messages = lines.splitlines()
         notify_users = re.search("Present company includes: ((?:%s ?)+)\." %
-                                 NAMES_RE, lines)
+                                 ic.NAMES_RE, lines)
         if notify_users:
-            self.notify_users.extend(notify_users.groups()[0].split(" "))
+            self.notify_users.extend(notify_users.groups()[0].split())
 
     def _start_managers(self, lines):
         self.client.run_command("set interface %s %s" %
@@ -358,16 +401,28 @@ class FICSMainConnection(FICSConnection):
         # avoid having to init the in a specific order, connect calls should
         # be moved to a "start" function, so all managers would be in
         # the connection object when they are called
-        self.lvm = ListAndVarManager(self)
-        self.em = ErrorManager(self)
-        self.glm = SeekManager(self)
-        self.bm = BoardManager(self)
-        self.fm = FingerManager(self)
-        self.nm = NewsManager(self)
-        self.om = OfferManager(self)
-        self.cm = ChatManager(self)
-        self.alm = AutoLogOutManager(self)
-        self.adm = AdjournManager(self)
+        if self.ICC:
+            self.lvm = ICCListAndVarManager(self)
+            self.em = ICCErrorManager(self)
+            self.glm = ICCSeekManager(self)
+            self.bm = ICCBoardManager(self)
+            self.cm = ICCChatManager(self)
+            self.adm = ICCAdjournManager(self)
+            self.fm = ICCFingerManager(self)
+            self.nm = ICCNewsManager(self)
+            self.om = ICCOfferManager(self)
+            self.alm = ICCAutoLogOutManager(self)
+        else:
+            self.lvm = ListAndVarManager(self)
+            self.em = ErrorManager(self)
+            self.glm = SeekManager(self)
+            self.bm = BoardManager(self)
+            self.cm = ChatManager(self)
+            self.adm = AdjournManager(self)
+            self.fm = FingerManager(self)
+            self.nm = NewsManager(self)
+            self.om = OfferManager(self)
+            self.alm = AutoLogOutManager(self)
         self.com = ConsoleManager(self)
         self.bm.start()
         self.players.start()
@@ -377,7 +432,7 @@ class FICSMainConnection(FICSConnection):
 
     def start_helper_manager(self, set_user_vars):
         # if guest accounts disabled we will handle players in the main connection
-        if self.FatICS or self.USCN:
+        if self.FatICS or self.USCN or self.ICC:
             self.client.run_command("set pin 1")
         else:
             self.client.run_command("iset allresults 1")
@@ -389,7 +444,10 @@ class FICSMainConnection(FICSConnection):
             self.client.run_command("set open 1")
             self.client.run_command("set gin 1")
             self.client.run_command("set availinfo 1")
-        self.hm = HelperManager(self, self)
+        if self.ICC:
+            self.hm = ICCHelperManager(self, self)
+        else:
+            self.hm = HelperManager(self, self)
 
         # disable setting iveriables from console
         self.client.run_command("iset lock 1")
@@ -402,7 +460,6 @@ class FICSHelperConnection(FICSConnection):
 
     def _start_managers(self, lines):
         # The helper just wants only player and game notifications
-        self.main_conn.lounge_loaded.wait()
         # set open 1 is a requirement for availinfo notifications
         self.client.run_command("set open 1")
         self.client.run_command("set seek 0")
@@ -412,13 +469,10 @@ class FICSHelperConnection(FICSConnection):
         self.client.run_command("set chanoff 1")
         self.client.run_command("set gin 1")
         self.client.run_command("set availinfo 1")
-        if self.FatICS or self.USCN:
+        if self.FatICS or self.USCN or self.ICC:
             self.client.run_command("set pin 1")
         else:
             self.client.run_command("iset allresults 1")
             # ivar pin: http://www.freechess.org/Help/HelpFiles/new_features.html
             self.client.run_command("iset pin 1")
         self.hm = HelperManager(self, self.main_conn)
-
-        # disable setting iveriables from console
-        self.main_conn.client.run_command("iset lock 1")
