@@ -1,7 +1,6 @@
-from __future__ import print_function
 
 import re
-import threading
+import asyncio
 
 from gi.repository import GObject
 
@@ -18,7 +17,7 @@ from pychess.Utils.const import WHITEWON, WON_RESIGN, WON_DISCONNECTION, WON_CAL
     ABORTED, ABORTED_AGREEMENT, ABORTED_DISCONNECTION, ABORTED_EARLY, ABORTED_SERVER_SHUTDOWN, \
     ABORTED_ADJUDICATION, ABORTED_COURTESY, UNKNOWN_STATE, BLACK, WHITE, reprFile, \
     FISCHERRANDOMCHESS, CRAZYHOUSECHESS, WILDCASTLECHESS, WILDCASTLESHUFFLECHESS, ATOMICCHESS, \
-    LOSERSCHESS, SUICIDECHESS
+    LOSERSCHESS, SUICIDECHESS, reprResult
 
 from pychess.ic import IC_POS_INITIAL, IC_POS_ISOLATED, IC_POS_OP_TO_MOVE, IC_POS_ME_TO_MOVE, \
     IC_POS_OBSERVING, IC_POS_OBSERVING_EXAMINATION, IC_POS_EXAMINATING, GAME_TYPES, IC_STATUS_PLAYING, \
@@ -26,7 +25,7 @@ from pychess.ic import IC_POS_INITIAL, IC_POS_ISOLATED, IC_POS_OP_TO_MOVE, IC_PO
     BLKCMD_FLAG, parseRating
 
 from pychess.ic.FICSObjects import FICSGame, FICSBoard, FICSHistoryGame, \
-    FICSAdjournedGame, FICSJournalGame
+    FICSAdjournedGame, FICSJournalGame, FICSPlayer
 
 names = "(\w+)"
 titles = "((?:\((?:GM|IM|FM|WGM|WIM|WFM|TM|SR|TD|SR|CA|C|U|D|B|T|\*)\))+)?"
@@ -189,10 +188,9 @@ class BoardManager(GObject.GObject):
         'playGameCreated': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
         'obsGameCreated': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
         'exGameCreated': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
+        'exGameBackward': (GObject.SignalFlags.RUN_FIRST, None, (int, int)),
         'archiveGamePreview': (GObject.SignalFlags.RUN_FIRST, None,
                                (object, )),
-        'boardUpdate': (GObject.SignalFlags.RUN_FIRST, None,
-                        (int, int, int, str, str, str, str, int, int)),
         'timesUpdate': (GObject.SignalFlags.RUN_FIRST, None,
                         (int, int, int,)),
         'obsGameEnded': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
@@ -245,6 +243,10 @@ class BoardManager(GObject.GObject):
             "Match request does not fit formula for %s:" % names,
             "%s's formula: (.+)" % names)
 
+        self.connection.expect_line(
+            self.on_game_remove,
+            "\{Game (\d+) \(([A-Za-z]+) vs\. ([A-Za-z]+)\) ([A-Za-z']+ .+)\} (\*|1/2-1/2|1-0|0-1)$")
+
         if self.connection.USCN:
             self.connection.expect_n_lines(
                 self.onPlayGameCreated,
@@ -335,6 +337,8 @@ class BoardManager(GObject.GObject):
 
     @classmethod
     def parseStyle12(cls, line, castleSigns=None):
+        # <12> rnbqkb-r pppppppp -----n-- -------- ----P--- -------- PPPPKPPP RNBQ-BNR
+        # B -1 0 0 1 1 0 7 Newton Einstein 1 2 12 39 39 119 122 2 K/e1-e2 (0:06) Ke2 0
         fields = line.split()
 
         curcol = fields[8] == "B" and BLACK or WHITE
@@ -456,11 +460,15 @@ class BoardManager(GObject.GObject):
                 # game.game_type = GAME_TYPES["examined"]
             game = self.connection.games.get(game)
 
+            # For examined games we have to create move queues here
+            # because game will be started later by generalStart()
+            if not hasattr(game, "queue"):
+                game.queue = asyncio.Queue()
+
             # don't start new game in puzzlebot/endgamebot when they just reuse gameno
             if game.relation == IC_POS_OBSERVING_EXAMINATION or \
                     (game.board is not None and game.board.pgn == pgn):
-                self.emit("boardUpdate", gameno, ply, curcol, lastmove, fen,
-                          wname, bname, wms, bms)
+                game.queue.put_nowait((gameno, ply, curcol, lastmove, fen, wname, bname, wms, bms))
                 return
 
             game.relation = relation
@@ -468,7 +476,7 @@ class BoardManager(GObject.GObject):
             self.gamesImObserving[game] = wms, bms
 
             # start a new game now or after smoves
-            self.gamemodelStartedEvents[game.gameno] = threading.Event()
+            self.gamemodelStartedEvents[game.gameno] = asyncio.Event()
             if no_smoves:
                 self.emit("exGameCreated", game)
                 self.gamemodelStartedEvents[game.gameno].wait()
@@ -484,8 +492,15 @@ class BoardManager(GObject.GObject):
                         self.connection.stored_owner, game.opponent.name))
                 self.connection.client.run_command("forward 999")
         else:
-            self.emit("boardUpdate", gameno, ply, curcol, lastmove, fen, wname,
-                      bname, wms, bms)
+            if gameno in self.connection.games.games_by_gameno:
+                game = self.connection.games.get_game_by_gameno(gameno)
+                if not hasattr(game, "queue"):
+                    game.queue = asyncio.Queue()
+                game.queue.put_nowait((gameno, ply, curcol, lastmove, fen, wname, bname, wms, bms))
+            else:
+                # In some cases (like lost on time) the last move is resent by FICS
+                # but game was already removed from self.connection.games
+                log.debug("Got %s but %s not in connection.games" % (style12, gameno))
 
     def onGameModelStarted(self, gameno):
         self.gamemodelStartedEvents[gameno].set()
@@ -545,8 +560,7 @@ class BoardManager(GObject.GObject):
         wplayer = self.connection.players.get(wname)
         bplayer = self.connection.players.get(bname)
         for player, rating in ((wplayer, wrating), (bplayer, brating)):
-            if game_type.rating_type in player.ratings and \
-                    player.ratings[game_type.rating_type] != rating:
+            if player.ratings[game_type.rating_type] != rating:
                 player.ratings[game_type.rating_type] = rating
                 player.emit("ratings_changed", game_type.rating_type, player)
 
@@ -576,7 +590,7 @@ class BoardManager(GObject.GObject):
                 player.game = game
 
         self.theGameImPlaying = game
-        self.gamemodelStartedEvents[gameno] = threading.Event()
+        self.gamemodelStartedEvents[gameno] = asyncio.Event()
         self.connection.client.run_command("follow")
         self.emit("playGameCreated", game)
 
@@ -864,8 +878,7 @@ class BoardManager(GObject.GObject):
         wplayer = self.connection.players.get(wname)
         bplayer = self.connection.players.get(bname)
         for player, rating in ((wplayer, wrating), (bplayer, brating)):
-            if game_type.rating_type in player.ratings and \
-                    player.ratings[game_type.rating_type] != rating:
+            if player.ratings[game_type.rating_type] != rating:
                 player.ratings[game_type.rating_type] = rating
                 player.emit("ratings_changed", game_type.rating_type, player)
         game = gameclass(wplayer,
@@ -889,6 +902,52 @@ class BoardManager(GObject.GObject):
 
         return game
 
+    def on_game_remove(self, match):
+        gameno, wname, bname, comment, result = match.groups()
+        result, reason = parse_reason(
+            reprResult.index(result),
+            comment,
+            wname=wname)
+
+        try:
+            wplayer = self.connection.players.get(wname)
+            wplayer.restore_previous_status()
+            # no status update will be sent by
+            # FICS if the player doesn't become available, so we restore
+            # previous status first (not necessarily true, but the best guess)
+        except KeyError:
+            print("%s not in self.connections.players - creating" % wname)
+            wplayer = FICSPlayer(wname)
+
+        try:
+            bplayer = self.connection.players.get(bname)
+            bplayer.restore_previous_status()
+        except KeyError:
+            print("%s not in self.connections.players - creating" % bname)
+            bplayer = FICSPlayer(bname)
+
+        game = FICSGame(wplayer,
+                        bplayer,
+                        gameno=int(gameno),
+                        result=result,
+                        reason=reason)
+        if wplayer.game is not None:
+            game.rated = wplayer.game.rated
+        game = self.connection.games.get(game, emit=False)
+
+        def coro():
+            # sometimes the latest move of the game comes after on_game_remove
+            # we have wait a little here before removing game from connection.games
+            # to let pychess a chance to processed it
+            yield from asyncio.wait(0.1)
+            self.connection.games.game_ended(game)
+            # Do this last to give anybody connected to the game's signals a chance
+            # to disconnect from them first
+            wplayer.game = None
+            bplayer.game = None
+
+        asyncio.async(coro())
+
     def onObserveGameCreated(self, matchlist):
         log.debug("'%s'" % (matchlist[1].string),
                   extra={"task": (self.connection.username,
@@ -936,7 +995,7 @@ class BoardManager(GObject.GObject):
             game = self.connection.games.get(game)
             self.gamesImObserving[game] = wms, bms
 
-            self.gamemodelStartedEvents[game.gameno] = threading.Event()
+            self.gamemodelStartedEvents[game.gameno] = asyncio.Event()
             self.emit("obsGameCreated", game)
             self.gamemodelStartedEvents[game.gameno].wait()
         else:
@@ -963,7 +1022,7 @@ class BoardManager(GObject.GObject):
             self.gamesImObserving[game] = wms, bms
             self.queuedStyle12s[game.gameno] = []
             self.queuedEmits[game.gameno] = []
-            self.gamemodelStartedEvents[game.gameno] = threading.Event()
+            self.gamemodelStartedEvents[game.gameno] = asyncio.Event()
 
             # FICS doesn't send the move list after 'observe' and 'follow' commands
             self.connection.client.run_command("moves %d" % game.gameno)
@@ -1029,7 +1088,8 @@ class BoardManager(GObject.GObject):
                 self.gamemodelStartedEvents[game.gameno].wait()
             self.emit("curGameEnded", game)
             self.theGameImPlaying = None
-            del self.gamemodelStartedEvents[game.gameno]
+            if game.gameno in self.gamemodelStartedEvents:
+                del self.gamemodelStartedEvents[game.gameno]
 
         elif game in self.gamesImObserving:
             log.debug("BM.onGameEnd: %s: gamesImObserving" % game)
@@ -1038,13 +1098,8 @@ class BoardManager(GObject.GObject):
                 self.queuedEmits[game.gameno].append(
                     lambda: self.emit("obsGameEnded", game))
             else:
-                try:
-                    event = self.gamemodelStartedEvents[game.gameno]
-                except KeyError:
-                    pass
-                else:
-                    log.debug("BM.onGameEnd: %s: event.wait()" % game)
-                    event.wait()
+                if game.gameno in self.gamemodelStartedEvents:
+                    self.gamemodelStartedEvents[game.gameno].wait()
                 del self.gamesImObserving[game]
                 self.emit("obsGameEnded", game)
 
