@@ -1,18 +1,14 @@
 # -*- coding: UTF-8 -*-
 
-from __future__ import absolute_import
-from __future__ import print_function
 
-from gi.repository import GLib, Gtk, Gdk, GObject
-
-import threading
+from gi.repository import Gtk, Gdk, GObject
 
 from pychess.System import conf
 from pychess.Utils.Cord import Cord
 from pychess.Utils.Move import Move, parseAny, toAN
 from pychess.Utils.const import ARTIFICIAL, FLAG_CALL, ABORT_OFFER, LOCAL, TAKEBACK_OFFER, \
     ADJOURN_OFFER, DRAW_OFFER, RESIGNATION, HURRY_ACTION, PAUSE_OFFER, RESUME_OFFER, RUNNING, \
-    KING, DROP, DROP_VARIANTS, PAWN, QUEEN, SITTUYINCHESS, QUEEN_PROMOTION
+    DROP, DROP_VARIANTS, PAWN, QUEEN, SITTUYINCHESS, QUEEN_PROMOTION
 
 from pychess.Utils.logic import validate
 from pychess.Utils.lutils import lmove, lmovegen
@@ -22,11 +18,46 @@ from .PromotionDialog import PromotionDialog
 from .BoardView import BoardView, rect, join
 
 
+def play_or_add_move(view, board, move):
+    if board.board.next is None:
+        # at the end of variation or main line
+        if not view.shownIsMainLine():
+            # add move to existing variation
+            view.model.add_move2variation(
+                board, move, view.shown_variation_idx)
+            view.shown += 1
+        else:
+            # create new variation
+            new_vari = view.model.add_variation(board, (move, ))
+            view.setShownBoard(new_vari[-1])
+    else:
+        # inside variation or main line
+        if board.board.next.lastMove == move.move:
+            # replay mainline move
+            view.shown += 1
+        elif board.board.next.children:
+            # try to find this move in variations
+            for i, vari in enumerate(board.board.next.children):
+                for node in vari:
+                    if type(node) != str and node.lastMove == move.move:
+                        # replay variation move
+                        view.setShownBoard(node.pieceBoard)
+                        return
+            # create new variation
+            new_vari = view.model.add_variation(board, (move, ))
+            view.setShownBoard(new_vari[-1])
+        else:
+            # create new variation
+            new_vari = view.model.add_variation(board, (move, ))
+            view.setShownBoard(new_vari[-1])
+
+
 class BoardControl(Gtk.EventBox):
     """ Creates a BoardView for GameModel to control move selection,
         action menu selection and emits signals to let Human player
         make moves and emit offers.
         SetuPositionDialog uses setup_position=True to disable most validation.
+        When game_preview=True just do circles and arrows
     """
 
     __gsignals__ = {
@@ -34,9 +65,11 @@ class BoardControl(Gtk.EventBox):
         'action': (GObject.SignalFlags.RUN_FIRST, None, (str, object))
     }
 
-    def __init__(self, gamemodel, action_menu_items, setup_position=False):
+    def __init__(self, gamemodel, action_menu_items, setup_position=False, game_preview=False, setup_sub_fen=False):
         GObject.GObject.__init__(self)
         self.setup_position = setup_position
+        self.game_preview = game_preview
+        self.setup_sub_fen = setup_sub_fen
         self.view = BoardView(gamemodel, setup_position=setup_position)
         self.add(self.view)
         self.variant = gamemodel.variant
@@ -69,7 +102,6 @@ class BoardControl(Gtk.EventBox):
         self.cids.append(self.connect("leave_notify_event", self.leave_notify))
 
         self.selected_last = None
-        self.stateLock = threading.Lock()
         self.normalState = NormalState(self)
         self.selectedState = SelectedState(self)
         self.activeState = ActiveState(self)
@@ -94,6 +126,9 @@ class BoardControl(Gtk.EventBox):
         self.gamemodel_cids.append(gamemodel.connect("game_started", onGameStart))
         self.keybuffer = ""
 
+        self.pre_arrow_from = None
+        self.pre_arrow_to = None
+
     def _del(self):
         self.view.disconnect(self.view_cid)
         for cid in self.cids:
@@ -135,9 +170,8 @@ class BoardControl(Gtk.EventBox):
                                               self.view.shown_variation_idx)
         # Ask player for which piece to promote into. If this move does not
         # include a promotion, QUEEN will be sent as a dummy value, but not used
-        if promotion is None and board[
-                cord0].sign == PAWN and cord1.cord in board.PROMOTION_ZONE[
-                    color]:
+        if promotion is None and board[cord0].sign == PAWN and \
+                cord1.cord in board.PROMOTION_ZONE[color]:
             if self.variant.variant == SITTUYINCHESS:
                 # no promotion allowed if we have queen
                 if board.board.boards[color][QUEEN]:
@@ -168,6 +202,7 @@ class BoardControl(Gtk.EventBox):
                 self.view.shownIsMainLine() and \
                 self.view.model.boards[-1] == board and \
                 self.view.model.status == RUNNING:
+            # emit move
             if self.setup_position:
                 self.emit("piece_moved", (cord0, cord1), board[cord0].color)
             else:
@@ -175,13 +210,7 @@ class BoardControl(Gtk.EventBox):
                 if self.view.model.examined:
                     self.view.model.connection.bm.sendMove(toAN(board, move))
         else:
-            if board.board.next is None and not self.view.shownIsMainLine():
-                self.view.model.add_move2variation(
-                    board, move, self.view.shown_variation_idx)
-                self.view.shown += 1
-            else:
-                new_vari = self.view.model.add_variation(board, (move, ))
-                self.view.setShownBoard(new_vari[-1])
+            play_or_add_move(self.view, board, move)
 
     def actionActivate(self, widget, key):
         """ Put actions from a menu or similar """
@@ -213,42 +242,31 @@ class BoardControl(Gtk.EventBox):
             self.emit("action", RESUME_OFFER, None)
 
     def shownChanged(self, view, shown):
-        def do_shown_changed():
-            if self.view is None:
-                return
-            self.lockedPly = self.view.shown
-            self.possibleBoards[self.lockedPly] = self._genPossibleBoards(
-                self.lockedPly)
-            if self.view.shown - 2 in self.possibleBoards:
-                del self.possibleBoards[self.view.shown - 2]
-
-        GLib.idle_add(do_shown_changed)
+        if self.view is None:
+            return
+        self.lockedPly = self.view.shown
+        self.possibleBoards[self.lockedPly] = self._genPossibleBoards(
+            self.lockedPly)
+        if self.view.shown - 2 in self.possibleBoards:
+            del self.possibleBoards[self.view.shown - 2]
 
     def moves_undone(self, gamemodel, moves):
-        self.stateLock.acquire()
-        try:
-            self.view.selected = None
-            self.view.active = None
-            self.view.hover = None
-            self.view.dragged_piece = None
-            self.view.setPremove(None, None, None, None)
-            if not self.view.model.examined:
-                self.currentState = self.lockedNormalState
-        finally:
-            self.stateLock.release()
+        self.view.selected = None
+        self.view.active = None
+        self.view.hover = None
+        self.view.dragged_piece = None
+        self.view.setPremove(None, None, None, None)
+        if not self.view.model.examined:
+            self.currentState = self.lockedNormalState
 
     def game_ended(self, gamemodel, reason):
-        self.stateLock.acquire()
-        try:
-            self.selected_last = None
-            self.view.selected = None
-            self.view.active = None
-            self.view.hover = None
-            self.view.dragged_piece = None
-            self.view.setPremove(None, None, None, None)
-            self.currentState = self.normalState
-        finally:
-            self.stateLock.release()
+        self.selected_last = None
+        self.view.selected = None
+        self.view.active = None
+        self.view.hover = None
+        self.view.dragged_piece = None
+        self.view.setPremove(None, None, None, None)
+        self.currentState = self.normalState
 
         self.view.startAnimation()
 
@@ -262,82 +280,138 @@ class BoardControl(Gtk.EventBox):
     def setLocked(self, locked):
         do_animation = False
 
-        self.stateLock.acquire()
-        try:
-            if locked and self.isLastPlayed(self.getBoard()) and \
-                    self.view.model.status == RUNNING:
-                if self.view.model.status != RUNNING:
-                    self.view.selected = None
-                    self.view.active = None
-                    self.view.hover = None
-                    self.view.dragged_piece = None
-                    do_animation = True
+        if locked and self.isLastPlayed(self.getBoard()) and \
+                self.view.model.status == RUNNING:
+            if self.view.model.status != RUNNING:
+                self.view.selected = None
+                self.view.active = None
+                self.view.hover = None
+                self.view.dragged_piece = None
+                do_animation = True
 
-                if self.currentState == self.selectedState:
-                    self.currentState = self.lockedSelectedState
-                elif self.currentState == self.activeState:
-                    self.currentState = self.lockedActiveState
-                else:
-                    self.currentState = self.lockedNormalState
+            if self.currentState == self.selectedState:
+                self.currentState = self.lockedSelectedState
+            elif self.currentState == self.activeState:
+                self.currentState = self.lockedActiveState
             else:
-                if self.currentState == self.lockedSelectedState:
-                    self.currentState = self.selectedState
-                elif self.currentState == self.lockedActiveState:
-                    self.currentState = self.activeState
-                else:
-                    self.currentState = self.normalState
-        finally:
-            self.stateLock.release()
+                self.currentState = self.lockedNormalState
+        else:
+            if self.currentState == self.lockedSelectedState:
+                self.currentState = self.selectedState
+            elif self.currentState == self.lockedActiveState:
+                self.currentState = self.activeState
+            else:
+                self.currentState = self.normalState
 
         if do_animation:
             self.view.startAnimation()
 
     def setStateSelected(self):
-        self.stateLock.acquire()
-        try:
-            if self.currentState in (self.lockedNormalState,
-                                     self.lockedSelectedState,
-                                     self.lockedActiveState):
-                self.currentState = self.lockedSelectedState
-            else:
-                self.view.setPremove(None, None, None, None)
-                self.currentState = self.selectedState
-        finally:
-            self.stateLock.release()
+        if self.currentState in (self.lockedNormalState,
+                                 self.lockedSelectedState,
+                                 self.lockedActiveState):
+            self.currentState = self.lockedSelectedState
+        else:
+            self.view.setPremove(None, None, None, None)
+            self.currentState = self.selectedState
 
     def setStateActive(self):
-        self.stateLock.acquire()
-        try:
-            if self.currentState in (self.lockedNormalState,
-                                     self.lockedSelectedState,
-                                     self.lockedActiveState):
-                self.currentState = self.lockedActiveState
-            else:
-                self.view.setPremove(None, None, None, None)
-                self.currentState = self.activeState
-        finally:
-            self.stateLock.release()
+        if self.currentState in (self.lockedNormalState,
+                                 self.lockedSelectedState,
+                                 self.lockedActiveState):
+            self.currentState = self.lockedActiveState
+        else:
+            self.view.setPremove(None, None, None, None)
+            self.currentState = self.activeState
 
     def setStateNormal(self):
-        self.stateLock.acquire()
-        try:
-            if self.currentState in (self.lockedNormalState,
-                                     self.lockedSelectedState,
-                                     self.lockedActiveState):
-                self.currentState = self.lockedNormalState
-            else:
-                self.view.setPremove(None, None, None, None)
-                self.currentState = self.normalState
-        finally:
-            self.stateLock.release()
+        if self.currentState in (self.lockedNormalState,
+                                 self.lockedSelectedState,
+                                 self.lockedActiveState):
+            self.currentState = self.lockedNormalState
+        else:
+            self.view.setPremove(None, None, None, None)
+            self.currentState = self.normalState
 
     def button_press(self, widget, event):
+        if event.button == 3:
+            # first we will draw a circle
+            cord = self.currentState.point2Cord(event.x, event.y)
+            self.pre_arrow_from = cord
+            self.view.pre_circle = cord
+            self.view.redrawCanvas()
+        else:
+            # remove all circles and arrows
+            need_redraw = False
+            if self.view.arrows:
+                self.view.arrows.clear()
+                need_redraw = True
+            if self.view.circles:
+                self.view.circles.clear()
+                need_redraw = True
+            if self.view.pre_arrow is not None:
+                self.view.pre_arrow = None
+                need_redraw = True
+            if self.view.pre_circle is not None:
+                self.view.pre_circle = None
+                need_redraw = True
+            if need_redraw:
+                self.view.redrawCanvas()
+
+        if self.game_preview:
+            return
         return self.currentState.press(event.x, event.y, event.button)
 
     def button_release(self, widget, event):
+        if event.button == 3:
+            # remove or finalize circle/arrow as needed
+            cord = self.currentState.point2Cord(event.x, event.y)
+            if self.view.pre_circle == cord:
+                if cord in self.view.circles:
+                    self.view.circles.remove(cord)
+                else:
+                    self.view.circles.add(cord)
+                self.view.pre_circle = None
+
+            if self.view.pre_arrow is not None:
+                if self.view.pre_arrow in self.view.arrows:
+                    self.view.arrows.remove(self.view.pre_arrow)
+                else:
+                    self.view.arrows.add(self.view.pre_arrow)
+                self.view.pre_arrow = None
+
+            self.pre_arrow_from = None
+            self.pre_arrow_to = None
+            self.view.redrawCanvas()
+
+        if self.game_preview:
+            return
         return self.currentState.release(event.x, event.y)
 
     def motion_notify(self, widget, event):
+        to = self.currentState.point2Cord(event.x, event.y)
+        if self.pre_arrow_from is not None:
+            if to != self.pre_arrow_from:
+                # this will be an arrow
+                if self.pre_arrow_to is not None and to != self.pre_arrow_to:
+                    # first remove the old one
+                    self.view.pre_arrow = None
+                    self.view.redrawCanvas()
+
+                arrow = self.pre_arrow_from, to
+                if arrow != self.view.pre_arrow:
+                    # draw the new arrow
+                    self.view.pre_arrow = arrow
+                    self.view.pre_circle = None
+                    self.view.redrawCanvas()
+                    self.pre_arrow_to = to
+
+            elif self.view.pre_circle is None:
+                # back to circle
+                self.view.pre_arrow = None
+                self.view.pre_circle = to
+                self.view.redrawCanvas()
+
         return self.currentState.motion(event.x, event.y)
 
     def leave_notify(self, widget, event):
@@ -346,12 +420,16 @@ class BoardControl(Gtk.EventBox):
     def key_pressed(self, keyname):
         if keyname in "PNBRQKMFSOox12345678abcdefgh":
             self.keybuffer += keyname
+
         elif keyname == "minus":
             self.keybuffer += "-"
+
         elif keyname == "at":
             self.keybuffer += "@"
+
         elif keyname == "equal":
             self.keybuffer += "="
+
         elif keyname == "Return":
             color = self.view.model.boards[-1].color
             board = self.view.model.getBoardAtPly(
@@ -361,21 +439,20 @@ class BoardControl(Gtk.EventBox):
             except:
                 self.keybuffer = ""
                 return
+
             if validate(board, move):
-                if self.view.shownIsMainLine() and self.view.model.boards[
-                        -1] == board:
+                if (self.view.model.curplayer.__type__ == LOCAL or self.view.model.examined) and \
+                        self.view.shownIsMainLine() and \
+                        self.view.model.boards[-1] == board and \
+                        self.view.model.status == RUNNING:
+                    # emit move
                     self.emit("piece_moved", move, color)
+                    if self.view.model.examined:
+                        self.view.model.connection.bm.sendMove(toAN(board, move))
                 else:
-                    if board.board.next is None and not self.view.shownIsMainLine(
-                    ):
-                        self.view.model.add_move2variation(
-                            board, move, self.view.shown_variation_idx)
-                        self.view.shown += 1
-                    else:
-                        new_vari = self.view.model.add_variation(board,
-                                                                 (move, ))
-                        self.view.setShownBoard(new_vari[-1])
+                    play_or_add_move(self.view, board, move)
             self.keybuffer = ""
+
         elif keyname == "BackSpace":
             self.keybuffer = self.keybuffer[:-1] if self.keybuffer else ""
 
@@ -383,8 +460,7 @@ class BoardControl(Gtk.EventBox):
         possible_boards = []
         if self.setup_position:
             return possible_boards
-        if len(self.view.model.players) == 2 and self.view.model.isEngine2EngineGame(
-        ):
+        if len(self.view.model.players) == 2 and self.view.model.isEngine2EngineGame():
             return possible_boards
         curboard = self.view.model.getBoardAtPly(ply,
                                                  self.view.shown_variation_idx)
@@ -432,17 +508,10 @@ class BoardState:
             return False
 
         if self.parent.setup_position:
-            to_piece = self.getBoard()[cord1]
             # prevent moving pieces inside holding
             if (cord0.x < 0 or cord0.x > self.FILES - 1) and \
                     (cord1.x < 0 or cord1.x > self.FILES - 1):
                 return False
-            # prevent moving kings off board
-            elif self.getBoard()[cord0].piece == KING and (cord1.x < 0 or cord1.x > self.FILES - 1):
-                return False
-            # prevent taking enemy king
-            elif to_piece is not None and to_piece.piece == KING:
-                return False
             else:
                 return True
 
@@ -801,10 +870,12 @@ class LockedActiveState(LockedBoardState):
                 self.view.selected, cord):
             # In mixed locked selected/active state and user selects a valid premove cord
             board = self.getBoard()
-            if board[
-                    self.view.selected].sign == PAWN and cord.cord in board.PROMOTION_ZONE[
-                        1 - board.color]:
-                promotion = self.parent.getPromotion()
+            if board[self.view.selected].sign == PAWN and \
+                    cord.cord in board.PROMOTION_ZONE[1 - board.color]:
+                if conf.get("autoPromote", False):
+                    promotion = lmove.PROMOTE_PIECE(QUEEN_PROMOTION)
+                else:
+                    promotion = self.parent.getPromotion()
             else:
                 promotion = None
             self.view.setPremove(board[self.view.selected], self.view.selected,
@@ -818,10 +889,12 @@ class LockedActiveState(LockedBoardState):
                 self.view.active, cord):
             # User drags a piece to a valid premove square
             board = self.getBoard()
-            if board[
-                    self.view.active].sign == PAWN and cord.cord in board.PROMOTION_ZONE[
-                        1 - board.color]:
-                promotion = self.parent.getPromotion()
+            if board[self.view.active].sign == PAWN and \
+                    cord.cord in board.PROMOTION_ZONE[1 - board.color]:
+                if conf.get("autoPromote", False):
+                    promotion = lmove.PROMOTE_PIECE(QUEEN_PROMOTION)
+                else:
+                    promotion = self.parent.getPromotion()
             else:
                 promotion = None
             self.view.setPremove(self.getBoard()[self.view.active],
