@@ -1,5 +1,4 @@
-from __future__ import absolute_import
-
+import asyncio
 import math
 import random
 
@@ -8,15 +7,14 @@ from gi.repository import Gtk, Pango, GObject
 from pychess.Players.Human import Human
 from pychess.Players.engineNest import discoverer
 from pychess.System import uistuff, conf
-from pychess.System.idle_add import idle_add
 from pychess.Utils.GameModel import GameModel
 from pychess.Utils.IconLoader import load_icon, get_pixbuf
 from pychess.Utils.TimeModel import TimeModel
 from pychess.Utils.const import LOCAL, ARTIFICIAL, WHITE, BLACK, NORMALCHESS
 from pychess.Variants import variants
 from pychess.ic import ICLogon
-from pychess.widgets.ionest import game_handler
 from pychess.widgets import newGameDialog
+from pychess.perspectives import perspective_manager
 
 from .Background import giveBackground
 from .ToggleComboBox import ToggleComboBox
@@ -83,13 +81,16 @@ class TaskerManager(Gtk.Table):
             yield (next, 1 - next)
             next = first - (1 - next)
 
+    def on_size_allocate(self, widget, allocation):
+        window = self.get_window()
+        if window is not None:
+            window.invalidate_rect(self.get_allocation(), False)
+
     def packTaskers(self, *widgets):
         self.widgets = widgets
 
         for widget in widgets:
-            widget.connect(
-                "size-allocate",
-                lambda *a: self.get_window().invalidate_rect(self.get_allocation(), False))
+            widget.connect("size-allocate", self.on_size_allocate)
         root = math.sqrt(len(widgets))
         # Calculate number of rows
         rows = int(math.ceil(root))
@@ -178,7 +179,6 @@ class NewGameTasker(Gtk.Alignment):
         widgets["startButton"].connect("clicked", self.startClicked)
         self.widgets["opendialog1"].connect("clicked", self.openDialogClicked)
 
-    @idle_add
     def __initPlayerCombo(self, discoverer, widgets):
         combo = self.playerCombo
         combo.update(newGameDialog.smallPlayerItems[0])
@@ -221,10 +221,11 @@ class NewGameTasker(Gtk.Alignment):
                           (engine, 1 - color, difficulty,
                            variants[NORMALCHESS], 5 * 60, 0), name)
 
+        perspective = perspective_manager.get_perspective("games")
         if color == WHITE:
-            game_handler.generalStart(gamemodel, player0tup, player1tup)
+            asyncio.async(perspective.generalStart(gamemodel, player0tup, player1tup))
         else:
-            game_handler.generalStart(gamemodel, player1tup, player0tup)
+            asyncio.async(perspective.generalStart(gamemodel, player1tup, player0tup))
 
 big_start = load_icon(48, "stock_init", "gnome-globe", "applications-internet")
 
@@ -237,54 +238,36 @@ class InternetGameTasker(Gtk.Alignment):
         tasker.unparent()
         self.add(tasker)
 
-        def asGuestCallback(check):
-            names = ICLogon.get_user_names()
-            self.widgets["usernameEntry"].set_text(names[1]
-                                                   if check.get_active() else names[0])
-            self.widgets["passwordLabel"].set_sensitive(not check.get_active())
-            self.widgets["passwordEntry"].set_sensitive(not check.get_active())
+        if ICLogon.dialog is None:
+            ICLogon.dialog = ICLogon.ICLogon()
 
-        self.widgets["asGuestCheck"].connect("toggled", asGuestCallback)
-
-        uistuff.keep(self.widgets["asGuestCheck"], "asGuestCheck")
-
-        as_guest = self.widgets["asGuestCheck"]
-
-        def user_name_get_value(entry):
-            names = ICLogon.get_user_names()
-            if as_guest.get_active():
-                text = "%s|%s" % (names[0], entry.get_text())
-            else:
-                text = "%s|%s" % (entry.get_text(), names[1])
-            return text
-
-        def user_name_set_value(entry, value):
-            names = ICLogon.get_user_names(value=value)
-            if as_guest.get_active():
-                entry.set_text(names[1])
-            else:
-                entry.set_text(names[0])
-
-        uistuff.keep(self.widgets["usernameEntry"], "usernameEntry",
-                     user_name_get_value, user_name_set_value)
-        uistuff.keep(self.widgets["passwordEntry"], "passwordEntry")
+        liststore = Gtk.ListStore(str)
+        liststore.append(["FICS"])
+        liststore.append(["ICC"])
+        self.ics_combo = self.widgets["ics_combo"]
+        self.ics_combo.set_model(liststore)
+        renderer_text = Gtk.CellRendererText()
+        self.ics_combo.pack_start(renderer_text, True)
+        self.ics_combo.add_attribute(renderer_text, "text", 0)
+        self.ics_combo.connect("changed", ICLogon.dialog.on_ics_combo_changed)
+        self.ics_combo.set_active(conf.get("ics_combo", 0))
 
         self.widgets["connectButton"].connect("clicked", self.connectClicked)
         self.widgets["opendialog2"].connect("clicked", self.openDialogClicked)
 
         self.widgets["startIcon"].set_from_pixbuf(big_start)
 
+        uistuff.keep(self.widgets["ics_combo"], "icsCombo")
+        uistuff.keep(self.widgets["autoLogin"], "autoLogin")
+
     def openDialogClicked(self, button):
         ICLogon.run()
 
     def connectClicked(self, button):
-        asGuest = self.widgets["asGuestCheck"].get_active()
-        username = self.widgets["usernameEntry"].get_text()
-        password = self.widgets["passwordEntry"].get_text()
-
         ICLogon.run()
         if not ICLogon.dialog.connection:
-            ICLogon.dialog.widgets["logOnAsGuest"].set_active(asGuest)
-            ICLogon.dialog.widgets["nameEntry"].set_text(username)
-            ICLogon.dialog.widgets["passEntry"].set_text(password)
             ICLogon.dialog.widgets["connectButton"].clicked()
+
+
+new_game_tasker, internet_game_tasker = NewGameTasker(), InternetGameTasker()
+tasker.packTaskers(new_game_tasker, internet_game_tasker)
