@@ -1,18 +1,25 @@
-from __future__ import print_function
 import os
 import sys
+import shutil
+from collections import namedtuple
 
 from gi.repository import Gtk, Gdk, GLib, GObject
 from gi.repository.GdkPixbuf import Pixbuf
 
 from pychess.System import uistuff
 from pychess.System.prefix import getEngineDataPrefix
-from pychess.System.SubProcess import searchPath
 from pychess.Players.engineNest import discoverer, is_uci, is_cecp
 from pychess.widgets import newGameDialog
+from pychess.widgets import mainwindow
 
 firstRun = True
 
+VM = namedtuple('VM', 'name, ext, args')
+VM_LIST = [
+    VM("node", ".js", None),
+    VM("java", ".jar", "-jar"),
+    VM("python", ".py", "-u")]
+
 
 def run(widgets):
     global firstRun
@@ -104,8 +111,8 @@ class EnginesDialog():
                     self.cur_engine = engines[0]["name"]
                 engine = discoverer.getEngineByName(self.cur_engine)
                 options = engine.get("options")
+                self.options_store.clear()
                 if options:
-                    self.options_store.clear()
                     for option in options:
                         key = option["name"]
                         val = option
@@ -162,7 +169,7 @@ class EnginesDialog():
         # add button
         ################################################################
         engine_chooser_dialog = Gtk.FileChooserDialog(
-            _("Select engine"), None, Gtk.FileChooserAction.OPEN,
+            _("Select engine"), mainwindow(), Gtk.FileChooserAction.OPEN,
             (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN,
              Gtk.ResponseType.OK))
 
@@ -172,6 +179,9 @@ class EnginesDialog():
         filter.add_mime_type("application/x-ms-dos-executable")
         filter.add_mime_type("application/x-msdownload")
         filter.add_pattern("*.exe")
+        for vm in VM_LIST:
+            filter.add_pattern("*%s" % vm.ext)
+
         engine_chooser_dialog.add_filter(filter)
         self.add = False
 
@@ -181,12 +191,14 @@ class EnginesDialog():
 
             if response == Gtk.ResponseType.OK:
                 new_engine = engine_chooser_dialog.get_filename()
-                if new_engine.lower().endswith(
-                        ".exe") and sys.platform != "win32":
+                vm_name = None
+                vm_args = None
+                vmpath = ""
+                if new_engine.lower().endswith(".exe") and sys.platform != "win32":
                     vm_name = "wine"
-                    vmpath = searchPath(vm_name, access=os.R_OK | os.X_OK)
+                    vmpath = shutil.which(vm_name, mode=os.R_OK | os.X_OK)
                     if vmpath is None:
-                        msg_dia = Gtk.MessageDialog(type=Gtk.MessageType.ERROR,
+                        msg_dia = Gtk.MessageDialog(mainwindow(), type=Gtk.MessageType.ERROR,
                                                     buttons=Gtk.ButtonsType.OK)
                         msg_dia.set_markup(_("<big><b>Unable to add %s</b></big>" %
                                              new_engine))
@@ -194,15 +206,46 @@ class EnginesDialog():
                         msg_dia.run()
                         msg_dia.hide()
                         new_engine = ""
-                    else:
-                        vmpath += " "
-                else:
-                    vm_name = None
-                    vmpath = ""
+
+                for vm in VM_LIST:
+                    ext = os.path.splitext(new_engine)[1]
+                    if ext == vm.ext:
+                        vm_name = vm.name
+                        vm_args = vm.args
+                        vmpath = shutil.which(vm_name, mode=os.R_OK | os.X_OK)
+                        if vmpath is None:
+                            msg_dia = Gtk.MessageDialog(mainwindow(), type=Gtk.MessageType.ERROR,
+                                                        buttons=Gtk.ButtonsType.OK)
+                            msg_dia.set_markup(_("<big><b>Unable to add %s</b></big>" %
+                                                 new_engine))
+                            msg_dia.format_secondary_text(vm_name + _(" is not installed"))
+                            msg_dia.run()
+                            msg_dia.hide()
+                            new_engine = ""
+                        break
 
                 if new_engine:
+                    vm_ext_list = [vm.ext for vm in VM_LIST]
+                    if ext not in vm_ext_list and not os.access(new_engine, os.X_OK):
+                        msg_dia = Gtk.MessageDialog(mainwindow(), type=Gtk.MessageType.ERROR,
+                                                    buttons=Gtk.ButtonsType.OK)
+                        msg_dia.set_markup(_("<big><b>%s is not marked executable in the filesystem</b></big>" %
+                                             new_engine))
+                        msg_dia.format_secondary_text(_("Try chmod a+x %s" % new_engine))
+                        msg_dia.run()
+                        msg_dia.hide()
+                        self.add = False
+                        engine_chooser_dialog.hide()
+                        return
+
                     try:
-                        # Some engines support CECP and UCI, but variant engines are CECP,
+                        engine_command = []
+                        if vmpath:
+                            engine_command.append(vmpath)
+                        if vm_args is not None:
+                            engine_command.append(vm_args)
+                        engine_command.append(new_engine)
+                        # Some engines support CECP and UCI, but main variant engines are CECP,
                         # so we better to start with CECP this case
                         variant_engines = ("fmax", "sjaakii", "sjeng")
                         if any((True
@@ -211,9 +254,10 @@ class EnginesDialog():
                             checkers = [is_cecp, is_uci]
                         else:
                             checkers = [is_uci, is_cecp]
+
                         uci = False
                         for checker in checkers:
-                            check_ok = checker(vmpath + new_engine)
+                            check_ok = checker(engine_command)
                             if check_ok:
                                 uci = checker is is_uci
                                 break
@@ -225,9 +269,9 @@ class EnginesDialog():
                                     self.cur_engine)
                                 engine_chooser_dialog.set_filename(engine[
                                     "command"])
-                                msg_dia = Gtk.MessageDialog(
-                                    type=Gtk.MessageType.ERROR,
-                                    buttons=Gtk.ButtonsType.OK)
+                                msg_dia = Gtk.MessageDialog(mainwindow(),
+                                                            type=Gtk.MessageType.ERROR,
+                                                            buttons=Gtk.ButtonsType.OK)
                                 msg_dia.set_markup(
                                     _("<big><b>Unable to add %s</b></big>" %
                                       new_engine))
@@ -236,28 +280,32 @@ class EnginesDialog():
                                 msg_dia.run()
                                 msg_dia.hide()
                                 engine_chooser_dialog.hide()
+                                self.add = False
+                                engine_chooser_dialog.hide()
                                 return
+
                         binname = os.path.split(new_engine)[1]
                         for eng in discoverer.getEngines():
                             if eng["name"] == binname:
                                 binname = eng["name"] + "(1)"
                                 break
-                        self.widgets["engine_command_entry"].set_text(
-                            new_engine)
-                        self.widgets["engine_protocol_combo"].set_active(
-                            0 if uci else 1)
+
+                        self.widgets["engine_command_entry"].set_text(new_engine)
+                        self.widgets["engine_protocol_combo"].set_active(0 if uci else 1)
                         self.widgets["engine_args_entry"].set_text("")
 
                         # active = self.widgets["engine_protocol_combo"].get_active()
                         protocol = "uci" if uci else "xboard"
 
-                        discoverer.addEngine(binname, new_engine, protocol,
-                                             vm_name)
+                        if vm_args is not None:
+                            vm_args = vm_args.split(",")
+                        # print(binname, new_engine, protocol, vm_name, vm_args)
+                        discoverer.addEngine(binname, new_engine, protocol, vm_name, vm_args)
                         self.cur_engine = binname
                         self.add = False
                         discoverer.discover()
                     except:
-                        msg_dia = Gtk.MessageDialog(type=Gtk.MessageType.ERROR,
+                        msg_dia = Gtk.MessageDialog(mainwindow(), type=Gtk.MessageType.ERROR,
                                                     buttons=Gtk.ButtonsType.OK)
                         msg_dia.set_markup(_("<big><b>Unable to add %s</b></big>" %
                                              new_engine))
@@ -265,14 +313,32 @@ class EnginesDialog():
                             "There is something wrong with this executable"))
                         msg_dia.run()
                         msg_dia.hide()
+                        self.add = False
+                        engine_chooser_dialog.hide()
+                        return
                 else:
                     # restore the original
                     engine = discoverer.getEngineByName(self.cur_engine)
                     engine_chooser_dialog.set_filename(engine["command"])
+
             engine_chooser_dialog.hide()
 
         self.widgets["add_engine_button"].connect("clicked", add)
 
+        ################################################################
+        # vm args
+        ################################################################
+        def vm_args_changed(widget):
+            if self.cur_engine is not None:
+                new_args = self.widgets["vm_args_entry"].get_text().strip()
+                engine = discoverer.getEngineByName(self.cur_engine)
+                old_args = engine.get("vm_args")
+                if new_args != old_args:
+                    engine["vm_args"] = new_args.split()
+                    discoverer.save()
+
+        self.widgets["vm_args_entry"].connect("changed", vm_args_changed)
+
         ################################################################
         # engine args
         ################################################################
@@ -291,7 +357,7 @@ class EnginesDialog():
         # engine working directory
         ################################################################
         dir_chooser_dialog = Gtk.FileChooserDialog(
-            _("Select working directory"), None,
+            _("Select working directory"), mainwindow(),
             Gtk.FileChooserAction.SELECT_FOLDER,
             (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN,
              Gtk.ResponseType.OK))
@@ -321,12 +387,22 @@ class EnginesDialog():
                 engine = discoverer.getEngineByName(self.cur_engine)
                 old_protocol = engine["protocol"]
                 if new_protocol != old_protocol:
-                    engine_command = engine_chooser_dialog.get_filename()
+                    command = engine.get("command")
+                    engine_command = []
+                    vm_command = engine.get("vm_command")
+                    if vm_command is not None:
+                        engine_command.append(vm_command)
+                        vm_args = engine.get("vm_args")
+                        if vm_args is not None:
+                            engine_command.append(", ".join(vm_args))
+                    engine_command.append(command)
+
                     # is the new protocol supported by the engine?
                     if new_protocol == "uci":
                         check_ok = is_uci(engine_command)
                     else:
                         check_ok = is_cecp(engine_command)
+
                     if check_ok:
                         # discover engine options for new protocol
                         engine["protocol"] = new_protocol
@@ -360,16 +436,20 @@ class EnginesDialog():
                     self.widgets['remove_engine_button'].set_sensitive(False)
                 else:
                     self.widgets['remove_engine_button'].set_sensitive(True)
-                self.widgets["engine_command_entry"].set_text(engine[
-                    "command"])
+                self.widgets["engine_command_entry"].set_text(engine["command"])
                 engine_chooser_dialog.set_filename(engine["command"])
                 args = [] if engine.get("args") is None else engine.get("args")
                 self.widgets["engine_args_entry"].set_text(' '.join(args))
+
+                vm = engine.get("vm_command")
+                self.widgets["vm_command_entry"].set_text(vm if vm is not None else "")
+                args = [] if engine.get("vm_args") is None else engine.get("vm_args")
+                self.widgets["vm_args_entry"].set_text(' '.join(args))
+
                 directory = engine.get("workingDirectory")
                 dir_choice = directory if directory is not None else self.default_workdir
                 dir_chooser_dialog.set_current_folder(dir_choice)
-                self.widgets["engine_protocol_combo"].set_active(0 if engine[
-                    "protocol"] == "uci" else 1)
+                self.widgets["engine_protocol_combo"].set_active(0 if engine["protocol"] == "uci" else 1)
                 update_options()
                 self.selection = False
 
