@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-from __future__ import print_function
 
 import os
 from xml.dom import minidom
@@ -17,9 +15,10 @@ from .__init__ import NORTH, EAST, SOUTH, WEST, CENTER
 
 
 class PyDockTop(PyDockComposite, TabReceiver):
-    def __init__(self, id):
-        TabReceiver.__init__(self)
+    def __init__(self, id, perspective):
+        TabReceiver.__init__(self, perspective)
         self.id = id
+        self.perspective = perspective
         self.set_no_show_all(True)
         self.highlightArea = HighlightArea(self)
 
@@ -39,15 +38,17 @@ class PyDockTop(PyDockComposite, TabReceiver):
             ]
 
     def _del(self):
-        self.highlightArea.disconnect(self.highlightArea.cid)
+        if self.highlightArea.handler_is_connected(self.highlightArea.cid):
+            self.highlightArea.disconnect(self.highlightArea.cid)
+
         for button in self.buttons:
             for cid in self.button_cids[button]:
-                button.disconnect(cid)
+                if button.handler_is_connected(cid):
+                    button.disconnect(cid)
             button.myparent = None
+
         self.button_cids = {}
         self.highlightArea.myparent = None
-        #self.buttons = None
-        #self.highlightArea = None
 
         TabReceiver._del(self)
         PyDockComposite._del(self)
@@ -56,7 +57,7 @@ class PyDockTop(PyDockComposite, TabReceiver):
         return CENTER
 
     def __repr__(self):
-        return "top (%s) % self.id"
+        return "top (%s)" % self.id
 
     # ===========================================================================
     #    Component stuff
@@ -81,7 +82,7 @@ class PyDockTop(PyDockComposite, TabReceiver):
 
     def dock(self, widget, position, title, id):
         if not self.getComponents():
-            leaf = PyDockLeaf(widget, title, id)
+            leaf = PyDockLeaf(widget, title, id, self.perspective)
             self.addComponent(leaf)
             return leaf
         else:
@@ -107,6 +108,10 @@ class PyDockTop(PyDockComposite, TabReceiver):
     def __onDrop(self, arrowButton, sender):
         self.highlightArea.hide()
         child = sender.get_nth_page(sender.get_current_page())
+
+        for instance in sender.get_parent().getInstances(self.perspective):
+            instance.hideArrows()
+
         title, id = sender.get_parent().undock(child)
         self.dock(child, arrowButton.myposition, title, id)
 
@@ -218,9 +223,9 @@ class PyDockTop(PyDockComposite, TabReceiver):
         if parentElement.tagName in ("h", "v"):
             child1, child2 = children
             if parentElement.tagName == "h":
-                new = PyDockComposite(EAST)
+                new = PyDockComposite(EAST, self.perspective)
             else:
-                new = PyDockComposite(SOUTH)
+                new = PyDockComposite(SOUTH, self.perspective)
             new.initChildren(
                 self.__createWidgetFromXML(child1, idToWidget),
                 self.__createWidgetFromXML(child2, idToWidget),
@@ -242,7 +247,7 @@ class PyDockTop(PyDockComposite, TabReceiver):
         elif parentElement.tagName == "leaf":
             id = children[0].getAttribute("id")
             title, widget = idToWidget[id]
-            leaf = PyDockLeaf(widget, title, id)
+            leaf = PyDockLeaf(widget, title, id, self.perspective)
             for panelElement in children[1:]:
                 id = panelElement.getAttribute("id")
                 title, widget = idToWidget[id]
